import {
  Select,
  ModalService,
  Modal,
  GridEditable,
  Radio,
  Button,
  Form,
  IButtonEvent,
  TreeGridEditable,
  IComponentRender,
  DialogService,
  Tabs,
  Column,
  LoadingService,
} from '@zeedhi/common';
import {
  IDictionary,
  Metadata,
  IEventParam,
  Loader,
  Config,
  Http,
  Mask,
} from '@zeedhi/core';
import { PmeMask } from '@/config';

export class ImportacaoReceitaController {
  private appController = Loader.getInstance('AppController');

  private utilities = Loader.getInstance('Utilities');
  
  private utilitiesPLA = Loader.getInstance('UtilitiesPLA');

  private receitaController = Loader.getInstance('ReceitaController');

  private gridEditController = Loader.getInstance('GridEditableController');
  
  private dadosReceita = Loader.getInstance('Pla01300DadosReceitaController');
  
  private prato = Loader.getInstance('Pla01300PratoController');

  private modalImport!: Modal;

  private recipeDest = '';

  private prodDest: IDictionary = [];

  public get nmPlate() {
    let plate;
    let cdprato;
    if (Object.keys(this.receitaController.selectedPlate).length > 0) {
      plate = this.receitaController.selectedPlate;
      cdprato = Mask.getValueWithMask(plate.CDPRATO, this.utilities.maskPlate());
    } else {
      const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
      const currentRow = plateTree.datasource.currentRow;
      plate = currentRow;
      cdprato = Mask.getValueWithMask(currentRow.CDPRATO, this.utilities.maskPlate());
    }
    return `${cdprato} - ${plate.NMPRATO}`;
  }
  
  public async createImportModal() {
    this.modalImport = await ModalService.createFromJsonPost('',
      { jsonPath: 'planejamento/prato/modals/importacaoReceita' });
  }
  
  public validOpenModalImport({ component }: IEventParam<Button>) {
    const tabPratos: Tabs = Metadata.getInstance('tabPratos');
    let tipocard;
    let cdfilial;
    let cdserv;
    let cdPadrQuali;
    let cdRegiao;
    let grid: any;

    if (tabPratos.activeTab === 3) {
      const formFiltroTpCard = Metadata.getInstance<Form>('FORM_TIPOCARD');
      tipocard = formFiltroTpCard.value.TIPOCARD?.CDTIPOCARD;
      grid = Metadata.getInstance('RECIPE_TIPOCARD');

      if (tipocard === undefined) {
        this.utilities.showAlert('alert', 'O Tipo de Cardápio deve ser informado.', 'warning');
        DialogService.hide();
      } else if (!this.utilitiesPLA.alteraRecPadrao) {
        this.utilities.showAlert('alert', 'Bloqueados para Alteração', 'warning');
        DialogService.hide();
      } else this.showModalImportPlate(component);
    } else if (tabPratos.activeTab === 4) {
      const formFiltroUnidade = Metadata.getInstance<Form>('FORM_LOCAL');
      cdfilial = formFiltroUnidade.value.BRANCH_LOCAL?.CDFILIAL;
      grid = Metadata.getInstance('RECIPE_LOCAL');
      if (cdfilial === undefined) {
        this.utilities.showAlert('alert', 'Unidade deve ser informado.', 'warning');
        DialogService.hide();
      } else this.showModalImportPlate(component);
    } else if (tabPratos.activeTab === 5) {
      const formFiltroServico = Metadata.getInstance<Form>('FORM_SERVICE');
      cdfilial = formFiltroServico.value.BRANCH_SERVICE?.CDFILIAL;
      cdserv = formFiltroServico.value.SERVICE?.CDSERVFILI;
      grid = Metadata.getInstance('RECIPE_SERVICE');
      if (cdserv === undefined) {
        this.utilities.showAlert('alert', 'Unidade e serviço devem ser informados.', 'warning');
        DialogService.hide();
      } else this.showModalImportPlate(component);
    } else if (tabPratos.activeTab === 2) {
      grid = Metadata.getInstance('RECIPE_STANDARD');
      if (this.prato.operatorValid !== false && this.utilitiesPLA.alteraRecPadrao) this.showModalImportPlate(component);
      else {
        this.utilities.showAlert('Alert', 'Modificação em receita padrão depende de autorização.', 'warning');
        this.prato.openModalGroupOperador(component);
      }
    } else if (tabPratos.activeTab === 6) {
      const formFiltroPadrQuali = Metadata.getInstance<Form>('FORM_PADRAOQUALIDADE');
      cdPadrQuali = formFiltroPadrQuali.value.NMPADRAOQUALIDADE?.CDPADRAOQUALIDADE;
      grid = Metadata.getInstance('RECIPE_PADRAOQUALIDADE');
      if (cdPadrQuali === undefined) {
        this.utilities.showAlert('alert', 'O Padrão de Qualidade deve ser informado.', 'warning');
        DialogService.hide();
      } else this.showModalImportPlate(component);
    } else if (tabPratos.activeTab === 7) {
      const formFiltroRegiao = Metadata.getInstance<Form>('FORM_REGIAO');
      cdPadrQuali = formFiltroRegiao.value.NMPADRAOQUALIDADE?.CDPADRAOQUALIDADE;
      cdRegiao = formFiltroRegiao.value.NMREGIAO?.CDREGIAO;
      grid = Metadata.getInstance('RECIPE_REGION');
      if (cdPadrQuali === undefined || cdRegiao === undefined) {
        this.utilities.showAlert('alert', 'Padrão de Qualidade/Região devem ser informados.', 'warning');
        DialogService.hide();
      } else this.showModalImportPlate(component);
    }
    if (grid.editing) this.gridEditController.defaultState(grid);
    this.prodDest = grid.datasource.data;
    
    // this.setOriginPlate();
  }

  public async showModalImport({ component }: IDictionary) {
    const name = component.name;

    // Definição do destino da receita sem ternário aninhado
    if (name.includes('PADRAO')) this.recipeDest = 'PADRAO';
    else if (name.includes('TIPOCARD')) this.recipeDest = 'TIPOCARD';
    else if (name.includes('LOCAL')) this.recipeDest = 'LOCAL';
    else this.recipeDest = 'SERVICO';

    if (this.recipeDest === 'PADRAO') {
      // Se for PADRÃO, precisa validar o operador antes de abrir o modal
      if (this.receitaController.operatorValid === true && this.utilitiesPLA.alteraRecPadrao) {
        if (!this.modalImport) await this.createImportModal().then(() => this.modalImport.show());
        else await this.modalImport.show();
      } else {
        this.utilities.showAlert('Alert', 'Modificação em receita padrão depende de autorização.', 'warning');
        this.prato.openModalGroupOperador(component);
      }
    }
    
    if (this.recipeDest !== 'PADRAO') {
      if (!this.modalImport) await this.createImportModal().then(() => this.modalImport.show());
      else await this.modalImport.show();
    }

    this.setOriginPlate();

    const gridDest = this.utilities.returnGrid(component) as GridEditable;
    if (gridDest.editing) this.gridEditController.defaultState(gridDest);
    this.prodDest = gridDest.datasource.data;
  }

  public setType() { // verificar se está no prato
    if (Object.keys(this.receitaController.selectedPlate).length === 0) {
      if (this.prato.TCO === true) {
        const children = this.modalImport.childrenInstances;
        children.forEach((e: IDictionary) => {
          if (e.name === 'contendContainer') {
            const formChildren = e.childrenInstances;
            formChildren.forEach((ef: IDictionary) => {
              if (ef.name === 'formImport') {
                const padQuali = ef.children.find((name: any) => name.name === 'PADQUALIIMPORT');
                const regiao = ef.children.find((name: any) => name.name === 'NMREGIAOIMPORT');
                padQuali.isVisible = true;
                regiao.isVisible = true;
              }
            });
          }
        });
      }
      
      const tipoCard = Metadata.getInstance<Select>('TPCARD');
      const central = Metadata.getInstance<Select>('CENTRAL_IMPORT');
      if (this.prato.project === 'PLA') {
        tipoCard.isVisible = false;
        central.isVisible = false;
      } else {
        tipoCard.isVisible = true;
        central.isVisible = true;
      }
    }
  }
  
  public async showModalImportPlate(component: any) {
    if (!this.modalImport) {
      await this.createImportModal().then(() => {
        this.modalImport.show();
        this.setDataModalImport(component);
      });
    } else {
      await this.modalImport.show();
      this.setDataModalImport(component);
    }
  }
  
  public setDataModalImport(component: any) {
    this.setOriginPlate();
    this.setTypeCardPlate();
    const name = component.name;
    if (name.includes('PADRAO') > 0) this.recipeDest = 'PADRAO';
    else if (name.includes('TIPOCARD') > 0 || name.includes('TPCARD') > 0) this.recipeDest = 'TIPOCARD';
    else if (name.includes('LOCAL') > 0) this.recipeDest = 'LOCAL';
    else if (name.includes('PADQUALI') > 0) this.recipeDest = 'PADRAOQUALI';
    else if (name.includes('REGIAO') > 0) this.recipeDest = 'REGIAO';
    else this.recipeDest = 'SERVICO';
    // const gridDest = this.utilities.returnGrid(component) as GridEditable;
    // this.prodDest = gridDest.datasource.data;
  }
  
  public setTypeCardPlate() {
    if (this.modalImport && document.getElementsByName('TYPE').length > 0) {
      const type = Metadata.getInstance<Select>('TYPE');
      this.enableFieldsImport({ component: type });
    }
  }

  public destroyModalImport() {
    if (this.modalImport) {
      this.modalImport.destroy();
      this.modalImport = this.utilities.modalEmpty;
    }
  }
  
  public closeModalImport() {
    this.stateModalImport();
    const grid = Metadata.getInstance<GridEditable>('RECIPE');
    this.gridEditController.defaultState(grid);
    this.modalImport.destroy();
    this.modalImport = this.utilities.modalEmpty;
  }
  
  public stateModalImport() {
    const form = Metadata.getInstance<Form>('formImport');
    const type = Metadata.getInstance<Select>('TYPE');
    form.value.IMPORTTYPE = 'ACRESCENTAR';
    // form.value.CENTRAL = this.appController.getCdCentral();
    this.enableFieldsImport({ component: type });
  }

  public fullscreen({ component }: IButtonEvent) {
    const modal: IDictionary = Metadata.getInstance('modalHeaderImportacao');
    modal.parent.fullscreen = !modal.parent.fullscreen;
    component.iconName = (modal.parent.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
  }

  public async fieldDefaultValueImport({ component }: IEventParam<Select>) {
    if (Object.keys(this.receitaController.selectedPlate).length > 0) {
      if (component.name === 'CENTRAL_IMPORT') {
        component.datasource.data = [this.appController.getCdCentral()];
        component.setValue(this.appController.getCdCentral());
      } else if (component.name === 'TPCARD') {
        component.datasource.data = [this.receitaController.tpCard];
        component.setValue(this.receitaController.tpCard);
      } else if (component.name === 'CDFILIAL') {
        component.datasource.data = [this.receitaController.branch];
        component.setValue(this.receitaController.branch.CDFILIAL);
      } else if (component.name === 'CDSERVFILI') {
        component.datasource.data = [this.receitaController.service];
        component.setValue(this.receitaController.service.CDSERVFILI);
      } else if (component.name === 'CDPRATO') {
        component.datasource.data = [this.receitaController.selectedPlate];
        component.setValue(this.receitaController.selectedPlate);
      }
    } else {
      const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
      const currentRow = plateTree.datasource.currentRow;
      if (component.name === 'CDPRATO') {
        component.datasource.data = [currentRow];
        component.setValue(currentRow);
      }
    }
  }

  public enableFieldsImport({ component }: IEventParam<Select>) {
    // const form = component.parent as Form;
    // const central = Metadata.getInstance<Select>('CENTRAL');
    const central = Metadata.getInstance<Select>('CENTRAL_IMPORT');
    const tpCard = Metadata.getInstance<Select>('TPCARD');
    const cdfilial = Metadata.getInstance<Select>('CDFILIAL');
    const cdservfili = Metadata.getInstance<Select>('CDSERVFILI');
    const padraoQuali = Metadata.getInstance<Select>('PADQUALIIMPORT');
    const regiao = Metadata.getInstance<Select>('NMREGIAOIMPORT');
    
    tpCard.setValue(null);
    cdfilial.setValue(null);
    cdservfili.setValue(null);
    padraoQuali.setValue(null);
    regiao.setValue(null);
    central.disabled = true;
    // central.setValue(this.appController.getCdCentral());
    
    switch (component.value) {
      case 'PADRAO':
        tpCard.disabled = true;
        cdfilial.disabled = true;
        cdservfili.disabled = true;
        padraoQuali.disabled = true;
        regiao.disabled = true;
        break;
      case 'TIPOCARD':
        tpCard.disabled = false;
        cdfilial.disabled = true;
        cdservfili.disabled = true;
        padraoQuali.disabled = true;
        regiao.disabled = true;
        central.disabled = !this.prato.permiteSelecCentral;
        break;
      case 'LOCAL':
        tpCard.disabled = true;
        cdfilial.disabled = false;
        cdservfili.disabled = true;
        padraoQuali.disabled = true;
        regiao.disabled = true;
        break;
      case 'PADRAOQUALI':
        tpCard.disabled = true;
        cdfilial.disabled = true;
        cdservfili.disabled = true;
        regiao.disabled = true;
        padraoQuali.disabled = false;
        break;
      case 'REGIAO':
        tpCard.disabled = true;
        cdfilial.disabled = true;
        cdservfili.disabled = true;
        padraoQuali.disabled = false;
        regiao.disabled = false;
        break;
      default:
        tpCard.disabled = true;
        cdfilial.disabled = false;
        cdservfili.disabled = false;
        padraoQuali.disabled = true;
        regiao.disabled = true;
        break;
    }
    
    this.setFilterGrid();
  }

  public setPratoFilter({ component }: IEventParam<Select>) {
    component.datasource.filter = {
      NIVEL: (Config.masks.plateMask as PmeMask).nivelMax,
    };
    component.datasource.get();
  }

  public fieldChangeFilterGrid({ component }: IEventParam<Select>) {
    if (!component.disabled) {
      this.setFilterGrid();
    }
  }
  
  public setValueTypeRecipe({ component }: IEventParam<Select>) {
    const data = component?.datasource?.data;
    if (Object.keys(this.receitaController.selectedPlate).length === 0) {
      if (this.prato.project === 'PLA' && (data?.length === 4 || data?.length === 6)) {
        const newData = data?.filter((item) => item.value !== 'TIPOCARD');
        component.datasource.updateData(newData);
      } else if (this.prato.TCO === false) {
        if (data.length === 5) { // sem cardapio e sem as 2 receitas
          const newData = data.slice(0, 3);
          component.datasource.updateData(newData);
        } else if (data.length > 5) { // tem cardapio e sem as 2 receitas
          const newData = data.slice(0, 4);
          component.datasource.updateData(newData);
        }
      }
    } else if (data?.length === 5 || data?.length === 6) { // cardapio
      const newData = data.slice(0, 4);
      component.datasource.updateData(newData);
    }
  }

  public setFilterGrid() {
    const type = Metadata.getInstance<Select>('TYPE');
    const central = Metadata.getInstance<Select>('CENTRAL_IMPORT');
    const tpCard = Metadata.getInstance<Select>('TPCARD');
    const cdfilial = Metadata.getInstance<Select>('CDFILIAL');
    const cdservfili = Metadata.getInstance<Select>('CDSERVFILI');
    
    let cdprato;
    let cdfilaux;
    let cdpadraoqualidade;
    let cdregiao;
  
    const grid: GridEditable = Metadata.getInstance('RECIPE');
    const formImport = Metadata.getInstance<Form>('formImport');
    
    let conditions = [];
    
    if (formImport.value.PRATO_ORIGEM !== null) {
      cdprato = formImport.value.PRATO_ORIGEM.CDPRATO;
      cdfilaux = formImport.value.PRATO_ORIGEM.CDFILAUXPRAT;
    } else {
      cdprato = null;
      cdfilaux = null;
    }
    
    if (Object.keys(this.receitaController.selectedPlate).length === 0 && this.prato.TCO !== false) {
      if (this.utilitiesPLA.isMerenda()) cdpadraoqualidade = Metadata.getInstance<Select>('PADQUALIIMPORT');
      if (this.utilitiesPLA.isMerenda()) cdregiao = Metadata.getInstance<Select>('NMREGIAOIMPORT');
    }
    
    if (this.formIsValid(type.value)) {
      conditions = [
        {
          name: 'TIPO',
          operator: '=',
          value: type.value || null,
        },
        {
          name: 'CDPRATO',
          operator: '=',
          value: cdprato,
        },
        {
          name: 'CDFILAUXPRAT',
          operator: '=',
          value: cdfilaux,
        },
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: central.value || null,
        },
        {
          name: 'CDTIPOCARD',
          operator: '=',
          value: tpCard.value ? tpCard.value.CDTIPOCARD : null,
        },
        {
          name: 'CDFILIAL',
          operator: '=',
          value: cdfilial.value ? cdfilial.value.CDFILIAL : null,
        },
        {
          name: 'CDSERVFILI',
          operator: '=',
          value: cdservfili.value ? cdservfili.value.CDSERVFILI : null,
        },
        {
          name: 'NRPADRAOQUALIDADE',
          operator: '=',
          value: cdpadraoqualidade?.value ? cdpadraoqualidade?.value.NRPADRAOQUALIDADE : null,
        },
        {
          name: 'CDREGIAO',
          operator: '=',
          value: cdregiao?.value ? cdregiao?.value.CDREGIAO : null,
        },
      ];
      
      grid.datasource.filter = conditions;
      grid.datasource.get();
    } else {
      conditions = [
        {
          name: 'CDPRATO',
          operator: '=',
          value: null,
        },
        {
          name: 'CDFILAUXPRAT',
          operator: '=',
          value: null,
        },
      ];
      grid.datasource.filter = conditions;
      grid.datasource.data = [];
    }
    
    this.setVisibleColumns();
  }

  public clearDependentField({ component }: IDictionary) {
    if (component.value) {
      const tpCard: Select = Metadata.getInstance('TPCARD');
      const cdfilial: Select = Metadata.getInstance('CDFILIAL');
      const cdservfili: Select = Metadata.getInstance('CDSERVFILI');
      if (component.name === 'CENTRAL_IMPORT') {
        tpCard.setValue(null);
        cdfilial.setValue(null);
        cdservfili.setValue(null);
      } else {
        cdservfili.setValue(null);
      }

      this.setFilterGrid();
    } else {
      const grid = Metadata.getInstance<GridEditable>('RECIPE');
      grid.datasource.data = [];
    }
  }

  public calculateFieldsImport({ row, component }: IDictionary) {
    // if (!row.ORIGINAL) row.ORIGINAL = { ...row };
    row.EDITED = true;
    const nameField = component.name.split('_')[0];
    if (!row.OLDROW) row.OLDROW = { ...row };
    row[nameField] = component.value;

    if (nameField !== 'VRPEFATCOC') row.VRPEFATCOC = row.VRPEFATCOCAP - 100; // ganho coccao
    if (nameField !== 'VRPEFATCOCAP') row.VRPEFATCOCAP = 100 + row.VRPEFATCOC; // aprov. coccao
    
    if (nameField !== 'VRPEFATCOR') row.VRPEFATCOR = 100 - row.VRPEFATCORAP; // perda limpeza
    if (nameField !== 'VRPEFATCORAP') row.VRPEFATCORAP = 100 - row.VRPEFATCOR; // aprov. limpeza

    const qtProd = row.QTPRODPRAT;
    const qtBruta = row.QTBRUTAUNR;

    if (!this.receitaController.idqtliqubrut && nameField !== 'QTBRUTA') { // qtd bruta
      // row.QTBRUTA = qtProd - ((qtProd * row.VRPEFATCOR) / 100) + ((qtProd * row.VRPEFATCOC) / 100);
      let qtde = qtProd / (1 + (row.VRPEFATCOC / 100));
      qtde /= (1 - (row.VRPEFATCOR / 100));
      row.QTBRUTAUNR = this.utilities.roundFloat(qtde / (1 - (row.VRPERCDEGELO / 100)), 5);
    }
    if (this.receitaController.idqtliqubrut && nameField !== 'QTPRODPRAT') { // quantidade
      // row.QTPRODPRAT = qtBruta - ((qtBruta * row.VRPEFATCOR) / 100) + ((qtBruta * row.VRPEFATCOC) / 100);
      let qtPratG = qtBruta - ((qtBruta * row.VRPERCDEGELO) / 100);
      qtPratG += ((qtPratG * row.VRPEFATCOC) / 100);
      row.QTPRODPRAT = qtPratG - ((qtPratG * row.VRPEFATCOR) / 100);
    }
    
    row.QTBRUTA = row.VRFATOCONVES
      ? row.QTBRUTAUNR * row.VRFATOCONVES
      : row.QTBRUTAUNR;

    // const baseNutri = row.QTBRUTA - (row.QTBRUTA * (row.VRPEFATCOR / 100));
    const baseCalcNutri = row.QTBRUTA - (row.QTBRUTA * (row.VRPERCDEGELO / 100));
    row.BASECALCNUTR = baseCalcNutri - (baseCalcNutri * (row.VRPEFATCOR / 100));
    
    row.QTCALCNUTR = ((row.VRPEFATNUT * row.BASECALCNUTR) / 100);

    // row.QTUNIDEST = row.VRFATOCONVES ? row.QTBRUTA * row.VRFATOCONVES : row.QTBRUTA;
    
    row.QTUNIDEST = row.QTBRUTA;

    // const grid = Metadata.getInstance<GridEditable>('RECIPE');
    const grid = component.parent;
    grid.updateRow(row);
  }
  
  public validDifferentRecipes() {
    const form = Metadata.getInstance<Form>('formImport');
    let pratoD;
    const type = Metadata.getInstance<Select>('TYPE');
    if (this.prodDest.length > 0) {
      pratoD = this.prodDest[0];
    } else if (Object.keys(this.receitaController.selectedPlate).length > 0) {
      pratoD = this.receitaController.selectedPlate;
    } else {
      const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
      pratoD = plateTree.datasource.currentRow;
      if (this.recipeDest === 'TIPOCARD') {
        const formFiltroTpCard = Metadata.getInstance<Form>('FORM_TIPOCARD');
        pratoD.CDTIPOCARD = formFiltroTpCard.value.TIPOCARD?.CDTIPOCARD;
        pratoD.CDFILCENTPME = formFiltroTpCard.value.CENTRAL;
      } else if (this.recipeDest === 'SERVICO') {
        const formFiltroServico = Metadata.getInstance<Form>('FORM_SERVICE');
        pratoD.CDSERVFILI = formFiltroServico.value.SERVICE?.CDSERVFILI;
      } else if (this.recipeDest === 'PADRAOQUALI') {
        const formFiltroPadQual = Metadata.getInstance<Form>('FORM_PADRAOQUALIDADE');
        pratoD.NRPADRAOQUALIDADE = formFiltroPadQual.value.NMPADRAOQUALIDADE?.NRPADRAOQUALIDADE;
      } else if (this.recipeDest === 'REGIAO') {
        const formFiltroRegiao = Metadata.getInstance<Form>('FORM_REGIAO');
        pratoD.NRPADRAOQUALIDADE = formFiltroRegiao.value.NMPADRAOQUALIDADE?.NRPADRAOQUALIDADE;
        pratoD.CDREGIAO = formFiltroRegiao.value.NMREGIAO?.CDREGIAO;
      }
    }
    const pratoO = form.value.PRATO_ORIGEM;
    let valid = true;
    if (pratoD.CDPRATO === pratoO.CDPRATO && pratoD.CDFILAUXPRAT === pratoO.CDFILAUXPRAT) {
      if (this.recipeDest === type.value) {
        const filialO = form.value.CDFILIAL ? form.value.CDFILIAL.CDFILIAL : '';
        const servO = form.value.CDSERVFILI ? form.value.CDSERVFILI.CDSERVFILI : '';
        const tpCardO = form.value.TPCARD ? form.value.TPCARD.CDTIPOCARD : '';
        const cdfilcentPmeO = form.value.TPCARD ? form.value.TPCARD.CDFILCENTPME : '';
        const nrpadqO = form.value.PADQUALIIMPORT ? form.value.PADQUALIIMPORT.NRPADRAOQUALIDADE : '';
        const cdregiaoO = form.value.NMREGIAOIMPORT ? form.value.NMREGIAOIMPORT.CDREGIAO : '';
        switch (type.value) {
          case 'PADRAO':
            valid = false;
            break;
          case 'TIPOCARD':
            if (pratoD.CDTIPOCARD === tpCardO && pratoD.CDFILCENTPME === cdfilcentPmeO) {
              valid = false;
            }
            break;
          case 'LOCAL':
            if (pratoD.CDFILIAL === form.value.CDFILIAL.CDFILIAL) {
              valid = false;
            }
            break;
          case 'SERVICO':
            if (pratoD.CDFILIAL === filialO && pratoD.CDSERVFILI === servO) {
              valid = false;
            }
            break;
          case 'PADRAOQUALI':
            if (pratoD.NRPADRAOQUALIDADE === nrpadqO) {
              valid = false;
            }
            break;
          case 'REGIAO':
            if (pratoD.NRPADRAOQUALIDADE === nrpadqO && pratoD.CDREGIAO === cdregiaoO) {
              valid = false;
            }
            break;
          default:
            break;
        }
      }
    }
    return valid;
  }

  public saveImport() {
    const type = Metadata.getInstance<Select>('TYPE');
    const grid = Metadata.getInstance<GridEditable>('RECIPE');
    const form = Metadata.getInstance<Form>('formImport');
    const dadosReceita = Object.keys(this.receitaController.selectedPlate).length;
    
    if (this.formIsValid(type.value)) {
      const rows: IDictionary = grid.datasource.data;
      
      if (grid.editing !== true) {
        if (rows.length > 0) {
          if (this.validDifferentRecipes()) {
            const importType = Metadata.getInstance<Radio>('IMPORTTYPE');
            const filter: IDictionary = {};
            let validHour = true;
    
            filter.CDPRATO = form.value.PRATO_ORIGEM?.CDPRATO || null;
            filter.CDFILAUXPRAT = form.value.PRATO_ORIGEM?.CDFILAUXPRAT || null;
            filter.CDFILCENTPME = form.value.CENTRAL_IMPORT || null;
            filter.CDTIPOCARD = form.value.TPCARD?.CDTIPOCARD || null;
            filter.CDFILIAL = form.value.CDFILIAL?.CDFILIAL || null;
            filter.CDSERVFILI = form.value.CDSERVFILI?.CDSERVFILI || null;
            filter.CDPADRAOQUALIDADE = form.value.PADQUALIIMPORT?.CDPADRAOQUALIDADE || null;
            filter.CDREGIAO = form.value.NMREGIAOIMPORT?.CDREGIAO || null;
            
            if (dadosReceita > 0) {
              rows.forEach((e: IDictionary) => {
                e.IMPORTTYPE = importType.value;
                e.RECIPEDEST = this.recipeDest;
                e.TYPE = type.value;
                e.CDTIPOCARD = this.receitaController.tpCard.CDTIPOCARD;
                e.CDFILIAL = this.receitaController.branch.CDFILIAL;
                e.CDSERVFILI = this.receitaController.service.CDSERVFILI;
                e.CDPRATO = form.value.PRATO_ORIGEM.CDPRATO;
                e.CDFILAUXPRAT = form.value.PRATO_ORIGEM.CDFILAUXPRAT;
                e.CDFILCENTPME = this.appController.getCdCentral();
                e.ORIGEM = filter;
                if (e.HRTEMPPREP !== null) {
                  if (e.HRTEMPPREP.length === 4) {
                    e.HRTEMPPREP = `${e.HRTEMPPREP.substring(0, 2)}:${e.HRTEMPPREP.substring(2)}`;
                  }
                }
                if (e.HRTEMPPREP && !this.utilities.timeIsValid(e.HRTEMPPREP)) validHour = false;
                e.HRTEMPPREP = this.utilities.formatHour(e.HRTEMPPREP, false);
              });
            } else {
              const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
              const currentRow = plateTree.datasource.currentRow;
              const receita = this.dadosReceita.setNameTab;
              let filial: any;
              let servico: any;
              let cardapio: any;
              let padQuali: any;
              let regiao: any;
              let central: any;
              if (receita === 'subTabReceitaServico') {
                const formService = Metadata.getInstance<Form>('FORM_SERVICE');
                central = this.appController.getCdCentral();
                filial = formService.value.BRANCH_SERVICE.CDFILIAL;
                servico = formService.value.SERVICE.CDSERVFILI;
                cardapio = null;
                padQuali = null;
                regiao = null;
              } else if (receita === 'subTabReceitaLocal') {
                const formUnidLocal = Metadata.getInstance<Form>('FORM_LOCAL');
                central = this.appController.getCdCentral();
                filial = formUnidLocal.value.BRANCH_LOCAL.CDFILIAL;
                servico = null;
                cardapio = null;
                padQuali = null;
                regiao = null;
              } else if (receita === 'subTabReceitaTipoCard') {
                const formTpCard = Metadata.getInstance<Form>('FORM_TIPOCARD');
                cardapio = formTpCard.value.TIPOCARD.CDTIPOCARD;
                central = formTpCard.value.CENTRAL;
                filial = currentRow.CDFILIAL;
                servico = null;
                padQuali = null;
                regiao = null;
              } else if (receita === 'subTabReceitaPadraoQualidade') {
                const formPadQuali = Metadata.getInstance<Form>('FORM_PADRAOQUALIDADE');
                central = this.appController.getCdCentral();
                padQuali = formPadQuali.value.NMPADRAOQUALIDADE.NRPADRAOQUALIDADE;
                filial = currentRow.CDFILIAL;
                servico = null;
                cardapio = null;
                regiao = null;
              } else if (receita === 'subTabReceitaRegiao') {
                const formRegiao = Metadata.getInstance<Form>('FORM_REGIAO');
                central = this.appController.getCdCentral();
                padQuali = formRegiao.value.NMPADRAOQUALIDADE.NRPADRAOQUALIDADE;
                regiao = formRegiao.value.NMREGIAO.CDREGIAO;
                filial = currentRow.CDFILIAL;
                servico = null;
                cardapio = null;
              } else {
                central = this.appController.getCdCentral();
                filial = currentRow.CDFILIAL;
                cardapio = null;
                servico = null;
                padQuali = null;
                regiao = null;
              }
              rows.forEach((e: IDictionary) => {
                e.CDPRATO_D = currentRow.CDPRATO;
                e.IMPORTTYPE = importType.value;
                e.RECIPEDEST = this.recipeDest;
                e.TYPE = type.value;
                e.CDTIPOCARD = cardapio;
                e.CDFILIAL = filial;
                e.CDSERVFILI = servico;
                e.NRPADRAOQUALIDADE = padQuali;
                e.CDREGIAO = regiao;
                e.CDPRATO = form.value.PRATO_ORIGEM.CDPRATO;
                e.CDFILAUXPRAT = form.value.PRATO_ORIGEM.CDFILAUXPRAT;
                e.CDFILCENTPME = central;
                e.ORIGEM = filter;
                if (e.HRTEMPPREP !== null) {
                  if (e.HRTEMPPREP.length === 4) {
                    e.HRTEMPPREP = `${e.HRTEMPPREP.substring(0, 2)}:${e.HRTEMPPREP.substring(2)}`;
                  }
                }
                if (e.HRTEMPPREP && !this.utilities.timeIsValid(e.HRTEMPPREP)) validHour = false;
                e.HRTEMPPREP = this.utilities.formatHour(e.HRTEMPPREP, false);
              });
            }
            let pratoD;
            if (this.prodDest.length > 0) {
              pratoD = this.prodDest;
              this.prodDest.forEach((prato: any) => {
                prato.CDFILCENTPME = this.appController.getCdCentral();
              });
            } else if (Object.keys(this.receitaController.selectedPlate).length > 0) {
              pratoD = [this.receitaController.selectedPlate];
              pratoD.forEach((prato: any) => {
                prato.CDFILCENTPME = this.appController.getCdCentral();
              });
            } else {
              const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
              pratoD = {
                CDFILAUXPRAT: plateTree.datasource.currentRow.CDFILAUXPRAT,
                CDFILIAL: plateTree.datasource.currentRow.CDFILIAL,
                CDNVPRATO: plateTree.datasource.currentRow.CDNVPRATO,
                CDPRATO: plateTree.datasource.currentRow.CDPRATO,
                CDPRATONIVANTE: plateTree.datasource.currentRow.CDPRATONIVANTE,
                HASCHILD: plateTree.datasource.currentRow.HASCHILD,
                IDTIPO: plateTree.datasource.currentRow.IDTIPO,
                NIVANTE: plateTree.datasource.currentRow.NIVANTE,
                NMPRATO: plateTree.datasource.currentRow.NMPRATO,
                PK: plateTree.datasource.currentRow.PK,
                CDTIPOCARD: null,
                CDFILCENTPME: null,
                CDSERVFILI: null,
                NRPADRAOQUALIDADE: null,
                CDREGIAO: null,
              };
              
              if (this.recipeDest === 'TIPOCARD') {
                const formFiltroTpCard = Metadata.getInstance<Form>('FORM_TIPOCARD');
                pratoD.CDTIPOCARD = formFiltroTpCard.value.TIPOCARD?.CDTIPOCARD;
                pratoD.CDFILCENTPME = formFiltroTpCard.value.CENTRAL;
              } else if (this.recipeDest === 'SERVICO') {
                const formFiltroServico = Metadata.getInstance<Form>('FORM_SERVICE');
                pratoD.CDSERVFILI = formFiltroServico.value.SERVICE?.CDSERVFILI;
              } else if (this.recipeDest === 'PADRAOQUALI') {
                const formFiltroPadQual = Metadata.getInstance<Form>('FORM_PADRAOQUALIDADE');
                pratoD.NRPADRAOQUALIDADE = formFiltroPadQual.value.NMPADRAOQUALIDADE?.NRPADRAOQUALIDADE;
              } else if (this.recipeDest === 'REGIAO') {
                const formFiltroRegiao = Metadata.getInstance<Form>('FORM_REGIAO');
                pratoD.NRPADRAOQUALIDADE = formFiltroRegiao.value.NMPADRAOQUALIDADE?.NRPADRAOQUALIDADE;
                pratoD.CDREGIAO = formFiltroRegiao.value.NMREGIAO?.CDREGIAO;
              }
              pratoD = [pratoD];
            }
            const params = [rows, pratoD];
    
            if (!validHour) this.utilities.showAlert('alertHour', 'Hora Inválida.', 'warning');
            else this.callSave('/saveImportRecipe', params, grid);
            // this.gridEditController.defaultState(grid);
          } else {
            this.utilities.showAlert('alertWarning',
              'Receita de origem deve ser diferente da receita de destino.',
              'warning');
          }
        } else this.utilities.showAlert('alertWarning', 'Operação bloqueada! Prato não possui receita.', 'warning');
      } else this.utilities.showAlert('alertWarning', 'É preciso salvar os dados antes de importar.', 'warning');
    } else form.validate();
  }

  private callSave(url: string, params: IDictionary, grid: GridEditable) {
    // const button = Metadata.getInstance<Button>('saveImportButton');
    LoadingService.show(' ');
    Http.post(url, { requestType: 'Row', row: params }).then((response) => {
      if (response) {
        // this.editable = false;
        this.utilities.showAlert('alertSuccess', 'Importação realizada com sucesso!', 'green');
        if (grid.editing) this.gridEditController.defaultState(grid);
        let gridDest: GridEditable;
        if (this.recipeDest === 'PADRAO') gridDest = Metadata.getInstance<GridEditable>('RECIPE_STANDARD');
        else if (this.recipeDest === 'TIPOCARD') gridDest = Metadata.getInstance<GridEditable>('RECIPE_TIPOCARD');
        else if (this.recipeDest === 'LOCAL') gridDest = Metadata.getInstance<GridEditable>('RECIPE_LOCAL');
        else if (this.recipeDest === 'PADRAOQUALI') gridDest = Metadata.getInstance<GridEditable>('RECIPE_PADRAOQUALIDADE');
        else if (this.recipeDest === 'REGIAO') gridDest = Metadata.getInstance<GridEditable>('RECIPE_REGION');
        else gridDest = Metadata.getInstance<GridEditable>('RECIPE_SERVICE');

        gridDest.datasource.get();
        this.receitaController.callEventRecipeChange();
        this.modalImport.hide();
        // this.destroyModalImport();
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
    }).finally(() => {
      LoadingService.hide();
    });
  }

  private formIsValid(type: string) {
    let valid = true;
    const form = Metadata.getInstance<Form>('formImport');

    if (type === 'PADRAO') {
      if (!form.value.PRATO_ORIGEM) valid = false;
    } else if (type === 'TIPOCARD') {
      if (this.prato.permiteSelecCentral === false) form.value.CENTRAL_IMPORT = this.appController.getCdCentral();
      if (!form.value.CENTRAL_IMPORT || !form.value.TPCARD || !form.value.PRATO_ORIGEM) valid = false;
    } else if (type === 'LOCAL') {
      if (!form.value.CDFILIAL || !form.value.PRATO_ORIGEM) valid = false;
    } else if (type === 'PADRAOQUALI') {
      if (!form.value.PADQUALIIMPORT || !form.value.PRATO_ORIGEM) valid = false;
    } else if (type === 'REGIAO') {
      if (!form.value.PADQUALIIMPORT && !form.value.NMREGIAOIMPORT) valid = false;
    } else if (!form.value.CDFILIAL || !form.value.CDSERVFILI) valid = false;

    return valid;
  }

  public setFilterFilial() {
    const cdfilial: Select = Metadata.getInstance('CDFILIAL');
    const central: Select = Metadata.getInstance('CENTRAL');

    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: central.value || null,
      },
    ];
    cdfilial.datasource.filter = conditions;
    cdfilial.datasource.get();
  }
  
  public setFilterService() {
    const cdservfili: Select = Metadata.getInstance('CDSERVFILI');
    const cdfilial: Select = Metadata.getInstance('CDFILIAL');
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdfilial.value?.CDFILIAL || null,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: this.utilitiesPLA.isMerenda() ? 'M' : 'N',
      },
    ];
    cdservfili.datasource.filter = conditions;
    cdservfili.datasource.get();
  }

  public setFilterTpCard() {
    const form = Metadata.getInstance<Form>('formImport');
    const tpCard: Select = Metadata.getInstance('TPCARD');
    
    let central;
    if (this.prato.permiteSelecCentral === false) central = this.appController.getCdCentral();
    else central = form.value.CENTRAL_IMPORT;
    
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: central || null,
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
    tpCard.datasource.filter = conditions;
    tpCard.datasource.get();
  }
  
  public async setOriginPlate() {
    if (document.getElementsByName('PRATO_ORIGEM').length > 0) {
      const pratoOrigem = Metadata.getInstance<Select>('PRATO_ORIGEM');
      
      let unidade: any;
      if (this.utilitiesPLA.isMerenda() && Object.keys(this.receitaController.selectedPlate).length === 0) {
        unidade = Metadata.getInstance<Form>('formUnidade').value.UNIDADEFORM.CDFILIAL;
      } else unidade = this.appController.getCdCentral();
      const nivel: any = (Config.masks.plateMask as PmeMask).nivelMax;
  
      pratoOrigem.datasource.filter = [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: unidade,
        },
        {
          name: 'CDNVPRATO',
          operator: '=',
          value: nivel,
        },
      ];
      let plate: IDictionary;
      if (this.prodDest.length > 0) {
        plate = this.prodDest[0];
      } else if (Object.keys(this.receitaController.selectedPlate).length > 0) {
        plate = this.receitaController.selectedPlate;
      } else {
        const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
        plate = plateTree.datasource.currentRow;
      }
      await pratoOrigem.datasource.get().then(() => {
        const found = {
          PK: plate.CDPRATO.concat(plate.CDFILAUXPRAT),
          CDPRATO: plate.CDPRATO,
          CDFILAUXPRAT: plate.CDFILAUXPRAT,
          CDNVPRATO: nivel,
          CDNMPRATO: this.nmPlate,
        };
        pratoOrigem.setValue(found);
      });
    }
  }
  
  public setVisibleColumns() {
    const type = Metadata.getInstance<Select>('TYPE');
    const grid: GridEditable = Metadata.getInstance('RECIPE');
    const qtdeRequerida = this.utilities.returnColumn(grid, 'QTBRUTA') as Column;
    const tempo = this.utilities.returnColumn(grid, 'HRTEMPPREP') as Column;
    const diasLim = this.utilities.returnColumn(grid, 'QTLIMSEG') as Column;
    const apNutricional = this.utilities.returnColumn(grid, 'VRPEFATNUT') as Column;
    const qtdeCalcNutriente = this.utilities.returnColumn(grid, 'QTCALCNUTR') as Column;
    
    const dataType = type?.datasource?.data;
    if (Object.keys(this.receitaController.selectedPlate).length === 0) { // validacao ocorre somente em prato
      if ((this.prato.project === 'PLA' && (dataType?.length === 4 || dataType?.length === 6))) {
        dataType?.splice(1, 1);
      }
    }
    
    if (type.value === 'PADRAOQUALI' || type.value === 'REGIAO') {
      qtdeRequerida.isVisible = false;
      tempo.isVisible = false;
      diasLim.isVisible = false;
      apNutricional.isVisible = false;
      qtdeCalcNutriente.isVisible = false;
    } else {
      qtdeRequerida.isVisible = true;
      tempo.isVisible = true;
      diasLim.isVisible = true;
      apNutricional.isVisible = true;
      qtdeCalcNutriente.isVisible = true;
    }
  }
  
  public async saveEditedRows({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);
    let validLimNegative = true;
    
    rows.forEach((e: IDictionary) => {
      e.QTLIMSEG = e.QTLIMSEG || null;
      if (e.QTLIMSEG < 0) validLimNegative = false;
    });
    
    if (!validLimNegative) this.utilities.showAlert('diasLimSegNegative', 'Dias Lim. Seg. não pode ser negativo.', 'red');
    else if (grid.isGridValid(true)) {
      this.gridEditController.newRow = false;
      grid.editing = false;
      const toolbarSlot = grid.toolbarSlot;
      toolbarSlot.forEach((e: IComponentRender) => {
        if (e.name === 'cancelButton') e.disabled = !e.disabled;
        if (e.name === 'saveButton') e.disabled = !e.disabled;
      });
      this.gridEditController.isInlineEditing = grid.editing;
      const data = [...grid.datasource.data];
      await grid.datasource.updateData([]);
      await grid.datasource.updateData(data);
      grid.datasource.currentRow = {}; // para retirar a linha selecionada
    } else this.utilities.showAlert('alert', 'Verificar campos obrigatórios!', 'error');
  }
  
  public setRowCdApre({ row, component }: IEventParam<IDictionary>) {
    row.EDITED = true;
    row.CDAPRERECE = component.value?.CDAPRERECE || null;
    row.NMAPRERECE = component.value?.NMAPRERECE || null;
    const grid: GridEditable = component.parent;
    grid.updateRow(row);
  }
}
