import {
  Select,
  GridEditable,
  Button,
  Tabs,
  Form,
  DialogService,
  TreeGridEditable,
  LoadingService,
  ITabsBeforeChangeEventParam,
  ModalService,
  Modal,
} from '@zeedhi/common';
import {
  Http,
  IDictionary,
  Metadata,
  IEventParam,
  Loader,
  Config,
  IDatasource,
} from '@zeedhi/core';
import { PmeCurrencyMask, PmeMask } from '@/config';
// import { Pla01300PratoController } from './Pla01300PratoController';

export class ReceitaController {
  private appController = Loader.getInstance('AppController');

  private utilities = Loader.getInstance('Utilities');
  
  private utilitiesPLA = Loader.getInstance('UtilitiesPLA');

  private crudController = Loader.getInstance('CrudController');
  
  public modalGroupOp!: Modal;
  
  private paramsFields: IDictionary = {};

  public selectedPlate: IDictionary = {};

  public disabledFilter = true;

  public propChange = false;

  public disablePropChange = false;

  public eventRecipeChange!: Function;

  // para callDeleteRows
  
  private grid!: GridEditable;
  
  public gridEditController = Loader.getInstance('GridEditableController');

  private datasourceName = '';
  
  private checkedRows!: IDictionary;

  // campos de filtro

  public tpCard: IDictionary = {};

  public service: IDictionary = {};

  public branch: IDictionary = {};
  
  public operatorValid = false;
  
  public idbloaltrelo = '';

  // visibilidade dos campos da receita
  public get idqtliqubrut() {
    return this.paramsFields.IDQTLIQUBRUT === 'L';
  }
  
  public get idqtliqubrutPadrao() {
    if (this.utilitiesPLA.alteraRecPadrao) return this.idqtliqubrut;
    return false;
  }
  
  public get idqtliqubrutTpCard() {
    if (this.utilitiesPLA.alteraRecTpCard) return this.idqtliqubrut;
    return false;
  }

  public get idqt() {
    return !this.idqtliqubrut;
  }

  public get idaprerece() {
    // return this.paramsFields.IDAPRERECE === 'S';
    return true;
  }

  public get idganhococc() {
    // return this.paramsFields.IDGANHOCOCC === 'S';
    return true;
  }

  public get idaprovcocc() {
    // return this.paramsFields.IDAPROVCOCC === 'S';
    return true;
  }

  public get idperdalimp() {
    // return this.paramsFields.IDPERDALIMP === 'S';
    return true;
  }

  public get idaprovlimp() {
    // return this.paramsFields.IDAPROVLIMP === 'S';
    return true;
  }
  
  public get vrpercdegelo() {
    // return this.paramsFields.IDPERDADEGELO === 'S';
    return true;
  }

  public get idtempprep() {
    // return this.paramsFields.IDTEMPPREP === 'S';
    return true;
  }
  
  public get idutreqcompentd() {
    // return this.paramsFields.IDUTREQCOMPENTD === 'S';
    return true;
  }

  public get idutdiaslimseg() {
    // return this.paramsFields.IDUTDIASLIMSEG === 'S';
    return true;
  }
  
  public getVisibilityFields() {
    let unidade: string;
    if (document.getElementsByName('UNIDADEFORM').length > 0) {
      const unidadeForm = Metadata.getInstance<Select>('UNIDADEFORM');
      unidade = unidadeForm.value?.CDFILIAL || this.appController.getCdCentral();
    } else if (document.getElementsByName('aggregated').length > 0) {
      const aggregated = Metadata.getInstance<Select>('aggregated');
      unidade = aggregated.value?.CDFILIAL || this.appController.getCdCentral();
    } else {
      // unidade = this.appController.getCdCentral();
      unidade = this.utilities.getCdUnidadeLogada;
    }
    const row = { CDFILIAL: unidade };
    Http.post('/visibilityFieldsReceita', { requestType: 'Row', row }).then((response) => {
      this.paramsFields = response.data.dataset.result;
    });
  }

  // Funcoes receita
  public setFilterGrids() {
    if (document.getElementsByName('tabsRecipes').length > 0) {
      const tab = Metadata.getInstance<Tabs>('tabsRecipes');
      if (tab.activeTab === 0) this.setfilterRecipeStandard();
      else if (tab.activeTab === 1) this.setfilterRecipeTpCard();
      else if (tab.activeTab === 2) this.setFilterRecipeLocal();
      else this.setFilterRecipeService();
    }
  }
  
  public setfilterRecipeStandard() {
    const grid: GridEditable = Metadata.getInstance('RECIPE_STANDARD');
    const conditions = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: this.selectedPlate.CDPRATO,
      },
      {
        name: 'CDFILAUXPRAT',
        operator: '=',
        value: this.selectedPlate.CDFILAUXPRAT ? this.selectedPlate.CDFILAUXPRAT : 'P',
      },
    ];
    grid.datasource.filter = conditions;
    grid.datasource.get().then(() => {
      // this.loadStandardRecipeData(conditions);
      const addButton: Button = grid.getChildInstance('addButton');
      if (this.utilitiesPLA.alteraRecPadrao) addButton.disabled = false;
      // this.enabledTabs();
    });
    
    const row = {
      COLUMN_NAME: 'CDGROPERREPA',
      ALIAS_NAME: 'ALTRECEPADRAO',
    };
    
    Http.post('/prato/returnParamsAddPlate', { requestType: 'Row', row }).then((callback) => {
      const result = callback.data.dataset.permission.result;
      if (result === 'NOPERMISSION') this.operatorValid = false;
      else this.operatorValid = true;
    });
  }

  public setfilterRecipeTpCard() {
    if (document.getElementsByName('RECIPE_TIPOCARD').length > 0) {
      const grid: GridEditable = Metadata.getInstance('RECIPE_TIPOCARD');
      const form = Metadata.getInstance<Form>('formRecipeTpcard').value;
  
      const conditions = [
        {
          name: 'P_CDPRATO',
          operator: '=',
          value: this.selectedPlate.CDPRATO,
        },
        {
          name: 'P_CDFILAUXPRAT',
          operator: '=',
          value: this.selectedPlate.CDFILAUXPRAT ? this.selectedPlate.CDFILAUXPRAT : 'P',
        },
        {
          name: 'P_CDFILCENTPME',
          operator: '=',
          value: form.CENTRAL ? form.CENTRAL : this.appController.getCdCentral(),
        },
        {
          name: 'P_CDTIPOCARD',
          operator: '=',
          value: this.tpCard.CDTIPOCARD || null,
        },
      ];
  
      grid.datasource.filter = conditions;
      grid.datasource.get().then(() => {
        const addButton: Button = grid.getChildInstance('addButton');
        if (this.utilitiesPLA.alteraRecTpCard) addButton.disabled = false;
      });
    }
  }

  public setFilterRecipeLocal() {
    if (document.getElementsByName('RECIPE_LOCAL').length > 0) {
      const grid: GridEditable = Metadata.getInstance('RECIPE_LOCAL');
  
      const conditions = [
        {
          name: 'CDPRATO',
          operator: '=',
          value: this.selectedPlate.CDPRATO,
        },
        {
          name: 'CDFILIAL',
          operator: '=',
          value: this.branch.CDFILIAL || null,
        },
        {
          name: 'CDFILAUXPRAT',
          operator: '=',
          value: this.selectedPlate.CDFILAUXPRAT || 'P',
        },
      ];
      grid.datasource.filter = conditions;
      grid.datasource.get().then(() => {
        const addButton: Button = grid.getChildInstance('addButton');
        addButton.disabled = false;
      });
    }
  }

  public setFilterRecipeService() {
    if (document.getElementsByName('RECIPE_SERVICE').length > 0) {
      const grid: GridEditable = Metadata.getInstance('RECIPE_SERVICE');
      const conditions = [
        {
          name: 'CDPRATO',
          operator: '=',
          value: this.selectedPlate.CDPRATO,
        },
        {
          name: 'CDFILIAL',
          operator: '=',
          value: this.branch.CDFILIAL || null,
        },
        {
          name: 'CDSERVFILI',
          operator: '=',
          value: this.service.CDSERVFILI || null,
        },
        {
          name: 'CDFILAUXPRAT',
          operator: '=',
          value: this.selectedPlate.CDFILAUXPRAT || 'P',
        },
      ];
      grid.datasource.filter = conditions;
      grid.datasource.get().then(() => {
        const addButton: Button = grid.getChildInstance('addButton');
        addButton.disabled = false;
      });
    }
  }

  // Outras funcoes
  
  public setTabPlate() {
    if (document.getElementsByName('tabsRecipes').length > 0) {
      const tab = Metadata.getInstance<Tabs>('tabsRecipes');
      if (this.selectedPlate.IDRECEITA === 'T') tab.activeTab = 1;
      else if (this.selectedPlate.IDRECEITA === 'F') tab.activeTab = 2;
      else if (this.selectedPlate.IDRECEITA === 'S') tab.activeTab = 3;
      else tab.activeTab = 0;
    }
  }

  public async addRow({ component }: IDictionary) {
    // const prato = Pla01300PratoController.getInstance();
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    this.propChange = false;
    this.disablePropChange = true;
    
    if (!this.gridEditController.newRow) {
      const item: IDictionary = {};
      if (grid.name === 'RECIPE_STANDARD') {
        item.IDQTDEFIXPR = 'N';
        item.IDTPREQPRODPADR = 'P';
        item.IDUTCOMPENTDIRP = 'N';
        item.IDPRODBASEPR = 'N';
        item.VRPEFATCOCPR = 0;
        item.VRPEFATCOCPRAP = 100;
        item.VRPEFATCORPR = 0;
        item.VRPEFATCORPRAP = 100;
        item.VRPEFATNUTPR = 100;
        item.HRTEMPPREPPR = null;
        item.QTPRODPRAT = '';
        item.QTBRUTAPR = null;
        item.QTBRUTAPRUNR = null;
        item.QTCALCNUTRPR = null;
        item.BASECALCNUTR = 0;
      } else if (grid.name === 'RECIPE_TIPOCARD') {
        item.IDQTDEFIXPRTP = 'N';
        // item.IDTPREQPRODPADR = 'P';
        item.IDUTCOMPENTDIRP = 'N';
        item.IDPRODBASEPRTP = 'N';
        item.VRPEFATCOCPRTP = 0;
        item.VRPEFATCOCPRTPAP = 100;
        item.VRPEFATCORPRTP = 0;
        item.VRPEFATCORPRTPAP = 100;
        item.VRPEFATNUTPRTP = 100;
        item.HRTEMPPREPPRTP = null;
        item.QTPRODPRATTP = '';
        item.QTBRUTAPRTP = null;
        item.QTBRUTAPRUNRTP = null;
        item.QTCALCNUTRPRTP = null;
        item.BASECALCNUTR = 0;
      } else if (grid.name === 'RECIPE_LOCAL') {
        item.IDQTDEFIXLO = 'N';
        item.IDTPREQPRODFILI = 'P';
        item.IDUTCOMPENTDIRF = 'N';
        item.IDPRODBASELO = 'N';
        item.VRPEFATCOCLO = 0;
        item.VRPEFATCOCLOAP = 100;
        item.VRPEFATCORLO = 100;
        item.VRPEFATCORLOAP = 100;
        item.VRPEFATNUTLO = 100;
        item.HRTEMPPREPLO = null;
        item.QTPRODRECLOC = '';
        item.QTBRUTALO = null;
        item.QTBRUTALOUNR = null;
        item.QTCALCNUTRLO = null;
        item.BASECALCNUTR = 0;
      } else if (grid.name === 'RECIPE_SERVICE') {
        item.IDQTDEFIXSE = 'N';
        item.IDTPREQPRODSERV = 'P';
        item.IDUTCOMPENTDIRS = 'N';
        item.IDPRODBASESE = 'N';
        item.VRPEFATCOCSE = 0;
        item.VRPEFATCOCSEAP = 100;
        item.VRPEFATCORSE = 0;
        item.VRPEFATCORSEAP = 100;
        item.VRPEFATNUTSE = 100;
        item.HRTEMPPREPSE = null;
        item.QTPRODRECSER = '';
        item.QTBRUTASE = null;
        item.QTBRUTASEUNR = null;
        item.QTCALCNUTRSE = null;
        item.BASECALCNUTR = 0;
      } else if (grid.name === 'RECIPE_REGION') {
        item.IDPRODBASEREGIAO = 'N';
        item.IDQTDEFIXREGIAO = 'N';
        item.IDPRODBASEREGIAO = 'P';
        item.VRPEFATCOCREGIAO = 0;
        item.VRPEFATCOCLOAP = 100;
        item.VRPEFATCORREGIAO = 0;
        item.VRPEFATCORLOAP = 100;
        item.QTPRODRECREGIAO = null;
        item.QTBRUTAREGIAOUNR = null;
      } else {
        item.IDQTDEFIXPADQ = 'N';
        item.IDPRODBASEPADQ = 'P';
        item.IDPRODBASEPADQ = 'N';
        item.VRPEFATCOCPADQ = 0;
        item.VRPEFATCOCLOAP = 100;
        item.VRPEFATCORPADQ = 0;
        item.VRPEFATCORLOAP = 100;
        item.QTPRODRECPADQ = null;
        item.QTBRUTAPADQUNR = null;
      }
      item.VRPERCDEGELO = 0;
      item.PK = grid.datasource.data.length + 1;

      if (grid.name === 'RECIPE_STANDARD' && !(this.operatorValid !== false && this.utilitiesPLA.alteraRecPadrao)) {
        this.utilities.showAlert('Alert', 'Modificação em receita padrão depende de autorização.', 'warning');
        this.openModalGroupOperador();
        // prato.nameComponentOperador = component.name;
        return;
      }

      if (grid.name === 'RECIPE_TIPOCARD' && !this.utilitiesPLA.alteraRecTpCard) {
        this.utilities.showAlert('alert', 'Bloqueados para Alteração', 'warning');
        return;
      }
      
      if (this.selectedPlate.CDFILAUXPRAT === 'P') {
        const menuForm = Metadata.getInstance<Form>('menuForm');
        const paramPlaneja = menuForm.value.paramplaneja;
        if (paramPlaneja.IDBLOALTRELO === 'S') {
          this.utilities.showAlert('alert',
            'Operador não autorizado a incluir/excluir produto em receita revisada.', 'warning');
          return;
        }
      }
      
      if (this.modalGroupOp) this.modalGroupOp.destroy();
      
      item.__is_new = true;
      this.gridEditController.newRow = true;
      
      await grid.addNewRow(item, 'start');
      this.gridEditController.controlAddButton({ component: grid });
      
      if (!this.gridEditController.isInlineEditing) {
        this.gridEditController.onInlineEdit({ component: grid });
      }
    }
  }
  
  public openModalGroupOperador() {
    this.createModalGroupOperador().then(() => {
      this.modalGroupOp.show();
    });
  }
  
  public async createModalGroupOperador() {
    if (!this.modalGroupOp) {
      this.modalGroupOp = await ModalService.createFromJsonPost('',
        { jsonPath: 'planejamento/prato/modals/formGroupOperador' });
    } else {
      await this.modalGroupOp.destroy();
      this.modalGroupOp = await ModalService.createFromJsonPost('',
        { jsonPath: 'planejamento/prato/modals/formGroupOperador' });
    }
  }

  public deleteRecipeStandard({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const checkedRows = grid.selectedRows;

    if (checkedRows.length === 0) this.utilities.showAlert('alertWarning', 'Nenhum registro selecionado.', 'warning');
    else if (grid.datasource.data.length === checkedRows.length) {
      this.utilities.showAlert('alertError', 'Operação bloquada! Receita Padrão deve possuir pelo menos um item.', 'error');
    } else this.gridEditController.deleteCheckedRows({ component });
  }
  
  // Salvamento dos grids
  public saveRecipeStandard({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    // const form = Metadata.getInstance<Form>('formRecipeStandard');
    
    if (grid.isGridValid(true)) {
      const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);
      let ctrlEstoq = true;
      let active = true;
      let validHour = true;
      let validLimNegative = true;

      rows.forEach((e: IDictionary) => {
        e.CDPRATO = this.selectedPlate.CDPRATO;
        e.CDFILAUXPRAT = this.selectedPlate.CDFILAUXPRAT;
        // e.SGUNIDADE = e.SGUNIDPROD;
        e.QTLIMSEGPADR = e.QTLIMSEGPADR || null;
        if (e.QTLIMSEGPADR < 0) validLimNegative = false;
        e.QTCOMPLPRODPADR = e.QTCOMPLPRODPADR || 0;
        e.CDAPRERECE = e.CDAPRERECE || null;
        // if (e.__is_new) e.CDPRODUTO = e.CDPRODUTO;
        if (e.__is_new && e.IDCNTRESTOQ !== 'S') ctrlEstoq = false;
        if (e.__is_new && e.IDPRODATIV !== 'A') active = false;
        if (e.HRTEMPPREPPR && !this.utilities.timeIsValid(e.HRTEMPPREPPR)) validHour = false;
        e.HRTEMPPREPPR = this.utilities.formatHour(e.HRTEMPPREPPR, false);
        if (e.originalRow) delete e.originalRow;
        if (e.OLDROW) delete e.OLDROW;
        if (e.VRPERCDEGELO === undefined || e.VRPERCDEGELO === null) e.VRPERCDEGELO = 0;
      });

      if (!validLimNegative) this.utilities.showAlert('diasLimSegNegative', 'Dias Lim. Seg. não pode ser negativo.', 'red');
      else if (this.saveIsValid(validHour, ctrlEstoq, active)) this.callSave('/saveRecipeStandard', rows, grid);
    } else this.utilities.showAlert('alert', 'Verificar campos obrigatórios!', 'error');
  }

  public saveRecipeTpCard({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    const form = Metadata.getInstance<Form>('formRecipeTpcard');
    
    if (grid.isGridValid(true)) {
      const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);
      let ctrlEstoq = true;
      let active = true;
      let validHour = true;

      rows.forEach((e: IDictionary) => {
        e.CDTIPOCARD = this.tpCard.CDTIPOCARD;
        e.CDPRATO = this.selectedPlate.CDPRATO;
        e.CDFILAUXPRAT = this.selectedPlate.CDFILAUXPRAT;
        // e.SGUNIDADE = e.SGUNIDPROD;
        e.CDAPRERECE = e.CDAPRERECE || null;
        // if (e.__is_new) e.CDPRODUTO = e.CDPRODUTO;
        if (e.__is_new && e.IDCNTRESTOQ !== 'S') ctrlEstoq = false;
        if (e.__is_new && e.IDPRODATIV !== 'A') active = false;
        if (e.HRTEMPPREPPRTP && !this.utilities.timeIsValid(e.HRTEMPPREPPRTP)) validHour = false;
        e.HRTEMPPREPPRTP = this.utilities.formatHour(e.HRTEMPPREPPRTP, false);
        // e.CDFILCENTPME = this.appController.getCdCentral();
        e.CDFILCENTPME = form.value.CDCENTRAL !== undefined ? form.value.CDCENTRAL : this.appController.getCdCentral();
        if (e.originalRow) delete e.originalRow;
        if (e.OLDROW) delete e.OLDROW;
        if (e.VRPERCDEGELO === undefined || e.VRPERCDEGELO === null) e.VRPERCDEGELO = 0;
      });

      if (this.saveIsValid(validHour, ctrlEstoq, active)) this.callSave('/saveRecipeTpCard', rows, grid);
    } else this.utilities.showAlert('alert', 'Verificar campos obrigatórios!', 'error');
  }

  public saveRecipeLocal({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    // const form = Metadata.getInstance<Form>('formRecipeLocal');
    
    if (grid.isGridValid(true)) {
      // const rows = grid.getEditedRows();
      const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);
      let ctrlEstoq = true;
      let active = true;
      let validHour = true;
      let validLimNegative = true;

      rows.forEach((e: IDictionary) => {
        e.CDFILIAL = this.branch.CDFILIAL;
        e.CDPRATO = this.selectedPlate.CDPRATO;
        e.CDFILAUXPRAT = this.selectedPlate.CDFILAUXPRAT;
        // e.SGUNIDADE = e.SGUNIDPROD;
        e.QTLIMSEGFILI = e.QTLIMSEGFILI || null;
        if (e.QTLIMSEGFILI < 0) validLimNegative = false;
        e.QTCOMPLPRODFILI = e.QTCOMPLPRODFILI || 0;
        e.CDAPRERECE = e.CDAPRERECE || null;
        // if (e.__is_new) e.CDPRODUTO = e.CDPRODUTO;
        if (e.__is_new && e.IDCNTRESTOQ !== 'S') ctrlEstoq = false;
        if (e.__is_new && e.IDPRODATIV !== 'A') active = false;
        if (e.HRTEMPPREPLO && !this.utilities.timeIsValid(e.HRTEMPPREPLO)) validHour = false;
        e.HRTEMPPREPLO = this.utilities.formatHour(e.HRTEMPPREPLO, false);
        if (e.originalRow) delete e.originalRow;
        if (e.OLDROW) delete e.OLDROW;
        if (e.VRPERCDEGELO === undefined || e.VRPERCDEGELO === null) e.VRPERCDEGELO = 0;
      });
      
      if (!validLimNegative) this.utilities.showAlert('diasLimSegNegative', 'Dias Lim. Seg. não pode ser negativo.', 'red');
      else if (this.saveIsValid(validHour, ctrlEstoq, active)) this.callSave('/saveRecipeLocal', rows, grid);
    } else this.utilities.showAlert('alert', 'Verificar campos obrigatórios!', 'error');
  }

  public saveRecipeService({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    // const form = Metadata.getInstance<Form>('formRecipeService');
    
    if (grid.isGridValid(true)) {
      const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);
      let ctrlEstoq = true;
      let active = true;
      let validHour = true;
      let validLimNegative = true;

      rows.forEach((e: IDictionary) => {
        e.CDFILIAL = this.branch.CDFILIAL;
        e.CDSERVFILI = this.service.CDSERVFILI;
        e.CDPRATO = this.selectedPlate.CDPRATO;
        e.CDFILAUXPRAT = this.selectedPlate.CDFILAUXPRAT;
        // e.SGUNIDADE = e.SGUNIDADE;
        e.QTLIMSEGSERV = e.QTLIMSEGSERV || null;
        if (e.QTLIMSEGSERV < 0) validLimNegative = false;
        e.QTCOMPLPRODSERV = e.QTCOMPLPRODSERV || 0;
        e.CDAPRERECE = e.CDAPRERECE || null;
        // if (e.__is_new) e.CDPRODUTO = e.CDPRODUTO;
        if (e.__is_new && e.IDCNTRESTOQ !== 'S') ctrlEstoq = false;
        if (e.__is_new && e.IDPRODATIV !== 'A') active = false;
        if (e.HRTEMPPREPSE && !this.utilities.timeIsValid(e.HRTEMPPREPSE)) validHour = false;
        e.HRTEMPPREPSE = this.utilities.formatHour(e.HRTEMPPREPSE, false);
        if (e.originalRow) delete e.originalRow;
        if (e.OLDROW) delete e.OLDROW;
        if (e.VRPERCDEGELO === undefined || e.VRPERCDEGELO === null) e.VRPERCDEGELO = 0;
      });
      
      if (!validLimNegative) this.utilities.showAlert('diasLimSegNegative', 'Dias Lim. Seg. não pode ser negativo.', 'red');
      else if (this.saveIsValid(validHour, ctrlEstoq, active)) this.callSave('/saveRecipeService', rows, grid);
    } else this.utilities.showAlert('alert', 'Verificar campos obrigatórios!', 'error');
  }

  private saveIsValid(validHour: boolean, ctrlEstoq: boolean, active: boolean) {
    let result = true;
    if (!validHour) this.utilities.showAlert('alertHour', 'Hora Inválida.', 'warning');
    else if (!ctrlEstoq) {
      this.utilities.showAlert('alertEstoq', 'Produtos que não controlam estoque', 'warning');
    } else if (!active) {
      this.utilities.showAlert(
        'alertActive',
        'Operação bloqueada! Produto não cadastrado para unidade ou inativo.',
        'warning',
      );
    }

    if (!validHour || !ctrlEstoq || !active) result = false;
    return result;
  }

  private callSave(url: string, rows: IDictionary, grid: GridEditable) {
    LoadingService.show(' ');
    Http.post(url, { requestType: 'Row', row: rows }).then((response) => {
      if (response) {
        // this.editable = false;
        this.callEventRecipeChange();
        this.utilities.showAlert('alertSuccess', 'Salvo com sucesso!', 'green');
        this.gridEditController.defaultState(grid);
      }
    }).catch((error: string) => {
      if (error.indexOf('ORA-00001') >= 0) {
        this.utilities.showAlert('alert', 'Impossível adicionar. Registro já existe.', 'error');
      } else this.utilities.showAlert('alert', `pme::${error}`, 'error');
    }).finally(() => {
      this.disablePropChange = false;
      LoadingService.hide();
    });
  }

  // Filtro dos fields
  public setFilterBranch({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDOPERADOR',
        operator: '=',
        value: 'OPERADOR_LOGADO',
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: 'T',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public setFilterService({ component }: IEventParam<Select>) {
    const branch: Select = Metadata.getInstance('BRANCH_SERVICE');
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: branch.value?.CDFILIAL || this.branch.CDFILIAL,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: this.utilitiesPLA.isMerenda() ? '' : 'N',
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  // Outras funcoes
  
  public saveRecRegion() {
    // const form = Metadata.getInstance<Form>('formRecipeRegion');
    const grid: GridEditable = Metadata.getInstance('RECIPE_REGION');
    const editedRows = grid.getEditedRows() as IDictionary;
    
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    const formRegiao = Metadata.getInstance<Form>('FORM_REGIAO');
    
    const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);

    rows.forEach((e: IDictionary) => {
      e.NRRECEREGIAO = '';
      e.NRPADRAOQUALIDADE = formRegiao.value.NMPADRAOQUALIDADE.NRPADRAOQUALIDADE;
      e.CDPADRAOQUALIDADE = formRegiao.value.NMPADRAOQUALIDADE.CDPADRAOQUALIDADE;
      e.CDREGIAO = formRegiao.value.NMREGIAO.CDREGIAO;
      e.CDPRATO = currentRow.CDPRATO;
      // e.CDPRODUTO = e.CDPRODUTO;
      e.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
      e.QTBRUTAREGIAOUNR = editedRows[0].QTBRUTAREGIAOUNR;
      e.QTPRODRECREGIAO = editedRows[0].QTBRUTAREGIAOUNR;
      e.SGUNIDADE = e.SGUNIDPROD;
      e.CDFILCENTPME = this.appController.getCdCentral();
      if (e.originalRow) delete e.originalRow;
      if (e.OLDROW) delete e.OLDROW;
    });
    
    if (grid.isGridValid(true)) {
      if (editedRows.length > 0) {
        this.callSave('/saveRecipeRegion', rows.at(-1), grid);
      } else {
        // this.gridEditController.defaultState(grid);
        this.utilities.showAlert('alert', 'Nenhum registro editado.', 'yellow');
      }
    }
  }
  
  public saveRecPadq() {
    // const form = Metadata.getInstance<Form>('formRecipePadraoQualidade');
    const grid: GridEditable = Metadata.getInstance('RECIPE_PADRAOQUALIDADE');
    const editedRows = grid.getEditedRows() as IDictionary;
    
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    const formRegiao = Metadata.getInstance<Form>('FORM_PADRAOQUALIDADE');
    
    const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);

    rows.forEach((e: IDictionary) => {
      e.NRRECEPADQUALI = '';
      e.NRPADRAOQUALIDADE = formRegiao.value.NMPADRAOQUALIDADE.NRPADRAOQUALIDADE;
      e.CDPADRAOQUALIDADE = formRegiao.value.NMPADRAOQUALIDADE.CDPADRAOQUALIDADE;
      e.CDPRATO = currentRow.CDPRATO;
      // e.CDPRODUTO = e.CDPRODUTO;
      e.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
      e.QTBRUTAPADQUNR = editedRows[0].QTBRUTAPADQUNR;
      e.QTPRODRECREGIAO = editedRows[0].QTBRUTAPADQUNR;
      e.SGUNIDADE = e.SGUNIDPROD;
      e.CDFILCENTPME = this.appController.getCdCentral();
      if (e.originalRow) delete e.originalRow;
      if (e.OLDROW) delete e.OLDROW;
    });
    
    if (grid.isGridValid(true)) {
      if (editedRows.length > 0) {
        this.callSave('/saveRecipePadraoQualidade', rows.at(-1), grid);
      } else {
        // this.gridEditController.defaultState(grid);
        this.utilities.showAlert('alert', 'Nenhum registro editado.', 'yellow');
      }
    }
  }
  
  public configCalculateCustom({ row, component }: IDictionary) {
    const gridParent = component.parent;
    let typeFieldVR = '';
    let typeFieldQT = '';
    let required = '';
    if (gridParent.name === 'RECIPE_REGION') {
      typeFieldVR = 'REGIAO';
      typeFieldQT = 'RECREGIAO';
      required = 'REGIAOUNR';
    } else if (gridParent.name === 'RECIPE_PADRAOQUALIDADE') {
      typeFieldVR = 'PADQ';
      typeFieldQT = 'RECPADQ';
      required = 'PADQUNR';
    }

    const percentualChange = this.calculateFieldsCustom(row, component, typeFieldVR, typeFieldQT, required);
    
    if (this.propChange) {
      gridParent.datasource.data.forEach((e: IDictionary) => {
        if (row.CDPRODUTO !== e.CDPRODUTO && e[`IDQTDEFIX${typeFieldVR}`] !== 'S') {
          this.changeProportionalCustom(e, typeFieldVR, typeFieldQT, percentualChange);
        }
      });
    }
  }

  private calculateFieldsCustom(
    row: IDictionary, component: IDictionary, typeFieldVR: string,
    typeFieldQT: string, required: string,
  ) {
    row.EDITED = true;
    const nameField = component.name.split('_')[0];
    if (!row.OLDROW) row.OLDROW = { ...row };
    row[nameField] = component.value;

    if (nameField !== `VRPEFATCOC${typeFieldVR}`) row[`VRPEFATCOC${typeFieldVR}`] = row.VRPEFATCOCLOAP - 100; // ganho coccao
    if (nameField !== 'VRPEFATCOCLOAP') row.VRPEFATCOCLOAP = 100 + row[`VRPEFATCOC${typeFieldVR}`]; // aprov. coccao
    
    if (nameField !== `VRPEFATCOR${typeFieldVR}`) {
      // perda limpeza
      row[`VRPEFATCOR${typeFieldVR}`] = 100 - row.VRPEFATCORLOAP;
    }
    
    if (nameField !== 'VRPEFATCORLOAP') row.VRPEFATCORLOAP = 100 - row[`VRPEFATCOR${typeFieldVR}`]; // ap. limpeza

    const qtProd = row[`QTPROD${typeFieldQT}`];
    const qtBruta = row[`QTBRUTA${required}`];
    
    if (this.idqtliqubrut && nameField !== `QTPROD${typeFieldQT}`) { // quantidade
      // row[`QTPROD${typeFieldQT}`] = qtBruta - ((qtBruta * row[`VRPEFATCOR${typeFieldVR}`]) / 100)
      // + ((qtBruta * row[`VRPEFATCOC${typeFieldVR}`]) / 100);
      const qtPratG = qtBruta + ((qtBruta * row[`VRPEFATCOC${typeFieldVR}`]) / 100);
      row[`QTPROD${typeFieldQT}`] = qtPratG - ((qtPratG * row[`VRPEFATCOR${typeFieldVR}`]) / 100);
      
      // qtProd = row[`QTPROD${typeFieldQT}`];
    }
    
    if (!this.idqtliqubrut && nameField !== `QTBRUTA${required}`) { // qtd bruta
      // row[`QTBRUTA${required}`] = qtProd - ((qtProd * row[`VRPEFATCOR${typeFieldVR}`]) / 100)
      //   + ((qtProd * row[`VRPEFATCOC${typeFieldVR}`]) / 100);
      const qtde = qtProd / (1 + (row[`VRPEFATCOC${typeFieldVR}`] / 100));
      row[`QTBRUTA${required}`] = this.utilities.roundFloat(qtde / (1 - (row[`VRPEFATCOR${typeFieldVR}`] / 100)), 5);
    }

    // row[`QTPROD${typeFieldQT}`] = qtBruta - ((qtBruta * row[`VRPEFATCOR${typeFieldVR}`]) / 100) +
    // ((qtBruta * row[`VRPEFATCOC${typeFieldVR}`]) / 100);
    
    const grid = component.parent;
    grid.updateRow(row);
    
    // return row[`QTBRUTA${required}`];
    return row[`QTBRUTA${required}`] / row.OLDROW[`QTBRUTA${required}`];
  }
  
  private changeProportionalCustom(row: IDictionary, typeFieldVR: string, typeFieldQT: string, percentual: number) {
    if (!row.OLDROW) row.OLDROW = { ...row };
    const oldRow = row.OLDROW;
    row.EDITED = true;
    
    row[`QTPROD${typeFieldQT}`] = oldRow[`QTPROD${typeFieldQT}`] * percentual;
    row[`QTBRUTA${typeFieldVR}UNR`] = oldRow[`QTBRUTA${typeFieldVR}UNR`] * percentual;
  }
  
  public configCalculate({ row, component }: IDictionary) {
    const gridParent = component.parent;
    let typeFieldVR = '';
    let typeFieldQT = '';
    let bruta = '';
    if (gridParent.name === 'RECIPE_STANDARD') {
      typeFieldVR = 'PR';
      typeFieldQT = 'PRAT';
      bruta = 'PRUNR';
    } else if (gridParent.name === 'RECIPE_TIPOCARD') {
      typeFieldVR = 'PRTP';
      typeFieldQT = 'PRATTP';
      bruta = 'PRUNRTP';
    } else if (gridParent.name === 'RECIPE_LOCAL') {
      typeFieldVR = 'LO';
      typeFieldQT = 'RECLOC';
      bruta = 'LOUNR';
    } else if (gridParent.name === 'RECIPE_SERVICE') {
      typeFieldVR = 'SE';
      typeFieldQT = 'RECSER';
      bruta = 'SEUNR';
    }
    
    // row.NMPRODUTO = gridParent.NMPRODUTO.value;
    // row.NMPRODUTO = row.NMPRODUTOAUX;
    const percentualChange = this.calculateFields(row, component, typeFieldVR, typeFieldQT, bruta);

    if (this.propChange) {
      gridParent.datasource.data.forEach((e: IDictionary) => {
        if (row.CDPRODUTO !== e.CDPRODUTO && e[`IDQTDEFIX${typeFieldVR}`] !== 'S') {
          this.changeProportional(e, typeFieldVR, typeFieldQT, percentualChange);
        }
      });
    }
    
    // gridParent.updateRow(row);
  }
  
  private calculateFields(
    row: IDictionary, component: IDictionary, typeFieldVR: string,
    typeFieldQT: string, bruta: string,
  ) {
    // if (!row.ORIGINAL) row.ORIGINAL = { ...row };
    row.EDITED = true;
    const nameField = component.name.split('_')[0];
    if (!row.OLDROW) row.OLDROW = { ...row };
    row[nameField] = component.value;

    if (nameField !== `VRPEFATCOC${typeFieldVR}`) {
      // ganho coccao
      row[`VRPEFATCOC${typeFieldVR}`] = row[`VRPEFATCOC${typeFieldVR}AP`] - 100;
    }
    
    if (nameField !== `VRPEFATCOC${typeFieldVR}AP`) {
      // aprov. coccao
      row[`VRPEFATCOC${typeFieldVR}AP`] = 100 + row[`VRPEFATCOC${typeFieldVR}`];
    }
    
    if (nameField !== `VRPEFATCOR${typeFieldVR}`) {
      // perda limpeza
      row[`VRPEFATCOR${typeFieldVR}`] = 100 - row[`VRPEFATCOR${typeFieldVR}AP`];
    }
    
    if (nameField !== `VRPEFATCOR${typeFieldVR}AP`) {
      // aprov. limpeza
      row[`VRPEFATCOR${typeFieldVR}AP`] = 100 - row[`VRPEFATCOR${typeFieldVR}`];
    }

    const qtProd = row[`QTPROD${typeFieldQT}`];
    const qtBruta = row[`QTBRUTA${bruta}`];

    if (!this.idqtliqubrut && nameField !== `QTBRUTA${bruta}`) { // qtd bruta
      let qtde = qtProd / (1 + (row[`VRPEFATCOC${typeFieldVR}`] / 100));
      qtde /= (1 - (row[`VRPEFATCOR${typeFieldVR}`] / 100));
      row[`QTBRUTA${bruta}`] = this.utilities.roundFloat(qtde / (1 - (row.VRPERCDEGELO / 100)), 5);
    }
    if (this.idqtliqubrut && nameField !== `QTPROD${typeFieldQT}`) { // quantidade
      // row[`QTPROD${typeFieldQT}`] = qtBruta - ((qtBruta * row[`VRPEFATCOR${typeFieldVR}`]) / 100)
      // + ((qtBruta * row[`VRPEFATCOC${typeFieldVR}`]) / 100);
      let qtPratG = qtBruta - ((qtBruta * row.VRPERCDEGELO) / 100);
      qtPratG += ((qtPratG * row[`VRPEFATCOC${typeFieldVR}`]) / 100);
      row[`QTPROD${typeFieldQT}`] = qtPratG - ((qtPratG * row[`VRPEFATCOR${typeFieldVR}`]) / 100);
    }
    
    if (typeFieldVR === 'PRTP') {
      // row.QTBRUTAPRUNRTP = row[`QTBRUTAPRUNRTP`];
      row.QTBRUTAPRTPUNR = row.QTBRUTAPRUNRTP;
    }
    
    // row[`QTBRUTA${typeFieldVR}`] = row[`QTBRUTA${typeFieldVR}UNR`];
    
    // const perc = row.OLDROW[`QTBRUTA${bruta}`]
    //   ? row[`QTBRUTA${bruta}`] / row.OLDROW[`QTBRUTA${bruta}`]
    //   : 1;
    
    row[`QTBRUTA${typeFieldVR}`] = row.VRFATOCONVES
      ? row[`QTBRUTA${bruta}`] * row.VRFATOCONVES
      : row[`QTBRUTA${bruta}`];
    
    // const baseNutri = row[`QTBRUTA${typeFieldVR}`]
    //                     - (row[`QTBRUTA${typeFieldVR}`] * (row[`VRPEFATCOR${typeFieldVR}`] / 100));
    
    const baseCalcNutri = row[`QTBRUTA${typeFieldVR}`] - (row[`QTBRUTA${typeFieldVR}`] * (row.VRPERCDEGELO / 100));
    row.BASECALCNUTR = baseCalcNutri - (baseCalcNutri * (row[`VRPEFATCOR${typeFieldVR}`] / 100));
    
    row[`QTCALCNUTR${typeFieldVR}`] = ((row[`VRPEFATNUT${typeFieldVR}`] * row.BASECALCNUTR) / 100);
    
    row[`QTUNIDEST${typeFieldVR}`] = row[`QTBRUTA${typeFieldVR}`];

    const grid = component.parent;
    grid.updateRow(row);

    return row[`QTBRUTA${bruta}`] / row.OLDROW[`QTBRUTA${bruta}`];
  }

  private changeProportional(row: IDictionary, typeFieldVR: string, typeFieldQT: string, percentual: number) {
    if (!row.OLDROW) row.OLDROW = { ...row };
    const oldRow = row.OLDROW;
    row.EDITED = true;
    
    row[`QTBRUTA${typeFieldVR}`] = oldRow[`QTBRUTA${typeFieldVR}`] * percentual;
    row[`QTPROD${typeFieldQT}`] = oldRow[`QTPROD${typeFieldQT}`] * percentual;
    row[`QTBRUTA${typeFieldVR}UNR`] = oldRow[`QTBRUTA${typeFieldVR}UNR`] * percentual;

    if (typeFieldVR === 'PRTP') row.QTBRUTAPRUNRTP = oldRow.QTBRUTAPRUNRTP * percentual;

    row[`QTCALCNUTR${typeFieldVR}`] = oldRow[`QTCALCNUTR${typeFieldVR}`] * percentual;

    row[`QTUNIDEST${typeFieldVR}`] = oldRow[`QTUNIDEST${typeFieldVR}`] * percentual;
  }

  public setRowEdited({ row, component }: IDictionary) {
    row.EDITED = true;
    const nameField = component.name.split('_')[0];
    row[nameField] = component.value;
    const grid: GridEditable = component.parent;
    grid.updateRow(row);
  }
  
  public setRowCdApre({ row, component }: IDictionary) {
    row.EDITED = true;
    row.CDAPRERECE = component.value;
    const grid: GridEditable = component.parent;
    grid.updateCell(row.PK, 'NMAPRERECE', row.CDAPRERECE);
  }
  
  public setRowSgunidEdited({ row, component }: IDictionary) {
    this.setRowEdited({ row, component });
    const gridParent = component.parent;
    if (component.selectValue) {
      const vrfatoconves = component.selectValue.VRFATOCONVES ? component.selectValue.VRFATOCONVES : 1;
      row.VRFATOCONVES = vrfatoconves;
      
      let typeField = '';
      let bruta;
      
      if (gridParent.name === 'RECIPE_STANDARD') {
        bruta = 'PR';
        typeField = 'PRUNR';
      } else if (gridParent.name === 'RECIPE_TIPOCARD') {
        bruta = 'PRTP';
        typeField = 'PRUNRTP';
      } else if (gridParent.name === 'RECIPE_LOCAL') {
        bruta = 'LO';
        typeField = 'LOUNR';
      } else if (gridParent.name === 'RECIPE_SERVICE') {
        bruta = 'SE';
        typeField = 'SEUNR';
      } else if (gridParent.name === 'RECIPE_PADRAOQUALIDADE') {
        bruta = 'PADQ';
        typeField = 'PADQUNR';
      } else if (gridParent.name === 'RECIPE_REGION') {
        bruta = 'REGIAO';
        typeField = 'REGIAOUNR';
      } else if (gridParent.name === 'RECIPE') {
        bruta = '';
        typeField = 'UNR';
      }
      
      if (row[`QTBRUTA${bruta}`]) {
        row[`QTBRUTA${bruta}`] = (vrfatoconves) ? row[`QTBRUTA${typeField}`] * vrfatoconves : row[`QTBRUTA${typeField}`];
        row[`QTUNIDEST${bruta}`] = row[`QTBRUTA${bruta}`];
        const baseNutri = row[`QTBRUTA${bruta}`]
                        - (row[`QTBRUTA${bruta}`] * (row[`VRPEFATCOR${bruta}`] / 100));
        row[`QTCALCNUTR${bruta}`] = ((row[`VRPEFATNUT${bruta}`] * baseNutri) / 100);
      }
    }
  }

  public fieldDefaultValue({ component }: IEventParam<Select>) {
    if (component.disabled) {
      if (component.name === 'TIPOCARD') {
        component.datasource.data = [this.tpCard];
        component.setValue(this.tpCard);
      } else if (component.name === 'BRANCH_LOCAL' || component.name === 'BRANCH_SERVICE') {
        component.datasource.data = [this.branch];
        component.setValue(this.branch);
      } else if (component.name === 'SERVICE') {
        component.datasource.data = [this.service];
        component.setValue(this.service);
      }
    }
  }

  public getFilterProd() {
    return [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: this.branch.CDFILIAL,
      },
    ];
  }

  public filterSGUN({ row, component }: IDictionary) {
    const filter = [
      {
        name: 'CDPRODUTO',
        operator: '=',
        value: row.CDPRODUTO || null,
      },
      {
        name: 'SGUNIDADE',
        operator: '=',
        value: row.SGUNIDPROD || null,
      },
      {
        name: 'SGUNIDESTO',
        operator: '=',
        value: null,
      },
      {
        name: 'TSGUNIDADE',
        operator: '=',
        value: 'T',
      },
    ];
    component.datasource.filter = filter;
    component.datasource.get();
  }

  public filterApre({ row, component }: IDictionary) {
    const filter = [
      {
        name: 'P_CDPRODUTO',
        operator: '=',
        value: row.CDPRODUTO || null,
      },
    ];
    component.datasource.filter = filter;
    component.datasource.get();
  }

  public produtoComponentProps = {
    component: 'ZdSelect',
    menuMaxHeight: 150,
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValue: 'CDPRODUTO',
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'SGUNIDADE',
        columnOnGrid: 'SGUNIDADE',
      },
      {
        column: 'SGUNIDADE',
        columnOnGrid: 'SGUNIDPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
    ],
    cssClass: 'columnProdGridWidth',
    returnObject: true,
    datasource: {
      uniqueKey: 'CDARVPROD',
      route: '/find/pratoProdutoEstoque',
      type: 'rest',
      limit: 10,
      lazyLoad: false,
    },
    validations: {
      required: {
        message: 'Campo obrigatório',
      },
    },
    events: {
      change: '{{ReceitaController.setLimpParams}}',
    },
  };
  
  public produtoComponentPropsTeste = {
    component: 'ZdSelect',
    menuMaxHeight: 150,
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValue: 'CDPRODUTO',
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'SGUNIDADE',
        columnOnGrid: 'SGUNIDADE',
      },
      {
        column: 'SGUNIDADE',
        columnOnGrid: 'SGUNIDPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
    ],
    cssClass: 'columnProdGridWidth',
    returnObject: true,
    datasource: {
      uniqueKey: 'CDARVPROD',
      route: '/find/opcprodrec_produto_opcao',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'Campo obrigatório',
      },
    },
    events: {
      focus: '{{ReceitaController.setNivel}}',
      change: '{{ReceitaController.setLimpParams}}',
    },
  };
  
  public setNivel({ component }: IEventParam<Select>) {
    const nivelMax = (Config.masks.productMask as PmeMask).nivelMax;
  
    const filter = [
      {
        name: 'P_NIVEL',
        operator: '=',
        value: nivelMax,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }

  public setLimpParams({ component, row }: IDictionary) {
    const gridParent: GridEditable = component.parent;
    const value = component.value;
    let type = '';
    
    if (gridParent.name === 'RECIPE_STANDARD') type = 'PR';
    else if (gridParent.name === 'RECIPE_TIPOCARD') type = 'PRTP';
    else if (gridParent.name === 'RECIPE_LOCAL') type = 'LO';
    else if (gridParent.name === 'RECIPE_SERVICE') type = 'SE';
    else if (gridParent.name === 'RECIPE_REGION') type = 'REGIAO';
    else if (gridParent.name === 'RECIPE_PADRAOQUALIDADE') type = 'PADQ';
    
    if (value) {
      row.CDPRODUTO = value.CDPRODUTO;
      row.SGUNIDPROD = value.SGUNIDADE;
      const sgUnidade = gridParent.getComponent(row.PK, 'SGUNIDADE'); // busca select
      this.filterSGUN({ row, component: sgUnidade });
      row.IDCNTRESTOQ = value.IDCNTRESTOQ;
      row.IDPRODATIV = value.IDPRODATIV;
      row.SGUNIDADE = value.SGUNIDADE;
      // row.NMPRODUTO = value.NMPRODUTO;
      // row.NMPRODUTOAUX = value.NMPRODUTO;
      row[`VRPEFATCOR${type}`] = value.VRPEFATOCORR;
      row[`VRPEFATCOR${type}AP`] = value.APROVLIMPEZA;
      // gridParent.updateCell(row[uniqueKey], 'SGUNIDADE', value.SGUNIDADE);
      
      gridParent.updateRow(row);
    } else {
      row.SGUNIDPROD = '';
      row.SGUNIDADE = '';
      gridParent.updateRow(row);
    }
  }

  public cancelEditing({ component }: IDictionary) {
    this.disablePropChange = false;
    this.gridEditController.cancelEditing({ component });
  }

  public async deleteCheckedRows({ component }: IDictionary) {
    this.grid = this.utilities.returnGrid(component) as GridEditable;
    this.checkedRows = this.grid.selectedRows;
    if (this.checkedRows.length === 0) this.utilities.showAlert('alert', 'Nenhum registro selecionado.', 'warning');
    else if (this.selectedPlate.CDFILAUXPRAT === 'P') {
      const menuForm = Metadata.getInstance<Form>('menuForm');
      const paramPlaneja = menuForm.value.paramplaneja;
      if (paramPlaneja.IDBLOALTRELO === 'S') {
        this.utilities.showAlert('alert',
          'Operador não autorizado a incluir/excluir produto em receita revisada.', 'warning');
      }
    } else {
      const regexFind = /\/find/gm;
      this.datasourceName = (this.grid.datasource as IDatasource).route.replace(regexFind, '');
      DialogService.show({
        name: component.name,
        type: 'warning',
        title: 'Atenção',
        text: 'Deseja excluir os registros?',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'Sim',
            events: { click: () => this.callDeleteRows() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'Não',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }

  private callDeleteRows() {
    LoadingService.show(' ');
    this.crudController.deleteRows(this.checkedRows, this.datasourceName).then(() => {
      this.grid.cancelEditedRows();
      this.grid.datasource.get();
      this.grid.selectedRows = [];
      DialogService.hide();
      this.utilities.showAlert('alert', 'Excluído com sucesso!', 'success');
      this.callEventRecipeChange();
    }).catch((error: string) => {
      DialogService.hide();
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    }).finally(() => {
      LoadingService.hide();
    });
  }

  public callEventRecipeChange() {
    if (this.eventRecipeChange !== undefined && typeof this.eventRecipeChange === 'function') this.eventRecipeChange();
  }
  
  public fieldsMask = {
    decimalCharacter: (Config.masks.numberMask as PmeCurrencyMask).decimalCharacter,
    digitGroupSeparator: (Config.masks.numberMask as PmeCurrencyMask).digitGroupSeparator,
    decimalPlaces: 2,
    suffixText: '%',
  };
  
  // cancelar grid das abas de receitas em dados do prato
  public cancelEditingTabs({ component }: ITabsBeforeChangeEventParam) {
    const leavingTab = component.tabs[component.activeTab];
    const forms = leavingTab.childrenInstances.filter((el: any) => el.component === 'ZdForm');
    const index = forms
      .findIndex((form: Form) => form.childrenInstances.find((el: any) => el.component === 'ZdGridEditable') !== undefined);
    const grid = forms[index]?.childrenInstances.find((el: any) => el.component === 'ZdGridEditable');
    if (grid?.editing) this.gridEditController.cancelEditing({ component: grid });
  }
}
