import {
  Metadata, Mask, Config, Loader, IDictionary, Http, IEventParam, FormatterParserProvider, I18n, dayjs as moment,
  IDatasource,
} from '@zeedhi/core';
import {
  AlertService, Modal, Grid, Select, IGridEvent, Button, Progress, TextInput, DialogService, Date as ZdDate, GridEditable,
  ModalService, LoadingService, Column,
} from '@zeedhi/common';
import { PmeMask, PmeCurrencyMask } from '@/config';

export class Utilities {
  private modal!: Modal;

  private gridPeriodo!: Grid;
  
  private gridPeriodoImport!: Grid;

  public appController = Loader.getInstance('AppController');
  
  public dtProcessamento = '';
  
  public cdCentralLogada = '';
  
  public nmCentralLogada = '';
  
  public modalEmpty!: Modal;
  
  private isDragging = false;

  public get dtProcessFormatada() {
    let date = this.dtProcessamento || localStorage.getItem('DTPROCESSAMENTO') || '';
    if (date) {
      date = `${moment(date, 'DD/MM/YYYY').format(this.getFormatLocale())}`;
    }
    if (!date) {
      return null; // Retorna null se a data for vazia
    }
    return date;
  }

  public getLanguage() {
    return localStorage.getItem('LOGIN_LANGUAGE') || 'pt_br';
  }
  
  public get getCdCentralLogada() {
    return this.cdCentralLogada || localStorage.getItem('CDFILIALLOGADA') || '';
  }
  
  public get getNmCentralLogada() {
    return this.nmCentralLogada || localStorage.getItem('NMFILIALLOGADA') || '';
  }
  
  public get getNomeUnidadeLogada() {
    // const codigo = this.cdCentralLogada || localStorage.getItem('CDFILIALLOGADA') || '';
    const nome = this.nmCentralLogada || localStorage.getItem('NMFILIALLOGADA') || '';
    
    return nome;
  }
  
  public get getCdUnidadeLogada() {
    const codigo = this.cdCentralLogada || localStorage.getItem('CDFILIALLOGADA') || '';
    // const nome = this.nmCentralLogada || localStorage.getItem('NMFILIALLOGADA') || '';
    
    return codigo;
  }
  
  public get getNomeCdUnidadeLogada() {
    return this.getNomeUnidadeLogada && this.getCdUnidadeLogada
      ? `${this.getNomeUnidadeLogada} | ${this.getCdUnidadeLogada}`
      : this.getNomeUnidadeLogada || this.getCdUnidadeLogada || '';
  }
  
  public get getOperadorInfo() {
    const nmacesso = localStorage.getItem('NMACESSO') || '';
    const nome = localStorage.getItem('LOGIN_USER_NAME') || '';
    
    if (nmacesso) return nmacesso;
    return nome;
  }
  
  public get getCdOperador() {
    const cdoperador = localStorage.getItem('CDOPERADOR') || '';
    return cdoperador;
  }
  
  public get getNomeCdOperador() {
    return this.appController.userName && this.getCdOperador
      ? `${this.appController.userName} | ${this.getCdOperador}`
      : this.appController.userName || this.getCdOperador || '';
  }

  // Variáveis do modal de cardápio
  private cdtipoCard = '';
  
  private central = '';

  private paramsCard?: IDictionary;

  public dtIniCard = '';

  public dtFinCard = '';
  
  public dtIniPeriodo = '';

  public dtFinPeriodo = '';

  public descCard = '';
  
  public description = '';
  
  public nrCardPme = '';
  
  public clicks = 0;
  
  public lastCellClicked: EventTarget | null = null;
  
  // Seta o valor do filtro do PRATO de acordo com o name do component passado como parametro.
  public setFilterPrato(cdfilial: string, nmprato: string) {
    const prato = Metadata.getInstance<Select>(nmprato);
    prato.setValue(null);
    prato.datasource.filter = {
      CDFILIAL: cdfilial,
    };
  }
  
  // Verifica se o prato inicial é maior que o prato final.
  public returnPrat(pratoIni: string, pratoFin: string): boolean {
    if (pratoIni > pratoFin) {
      return true;
    }
    
    return false;
  }

  public getProcessDate(cdfilial: string) {
    const row = (cdfilial === '' || cdfilial === undefined) ? [this.appController.cdCentral] : [cdfilial];
    return Http.post('/dataProcessamento', { requestType: 'Row', row }).then((response) => { // eslint-disable-line
      return this.getBloqDate().then((bloqDate) => { // eslint-disable-line
        let processDate = response.data.dataset.processDate[0];
        const format = 'DD/MM/YYYY';

        if (processDate !== '') processDate = moment(processDate, format).add(parseInt(bloqDate, 10), 'd').format(format);
        
        return processDate;
      });
    });
  }
  
  private getBloqDate() {
    return Http.get('/paramgeral').then((paramgeral) => {
      let bloqDate = paramgeral.data.data[0].QTDIABLOQCAR;
      if (!bloqDate || bloqDate === '') bloqDate = 0;
      return bloqDate;
    });
  }
  
  // seta o valor de um componente ZdDate como sendo a data de processamento
  public setDataProcessamentoZdDate({ component }: IEventParam<ZdDate>) {
    this.getProcessDate('').then((processDate: string) => {
      // const dtFormatLocale = moment(processDate, 'DD/MM/YYYY').format(this.getFormatLocale());
      // const dtFormatVal = moment(dtFormatLocale, this.getFormatLocale()).format(component.displayFormat);
      component.value = processDate;
    });
  }
  
  public async getParamPlaneja(cdfilial?: string) {
    const filter: IDictionary = [];
    let dados;
    const filial = (cdfilial === '' || cdfilial === undefined) ? [this.appController.cdCentral] : [cdfilial];
    this.onMountedFilterFind(filter, 'CDFILIAL', '=', filial);
    let paramplaneja = await Http.post('/find/paramplaneja', { requestType: 'FilterData', filter });
    if (paramplaneja.data.data.length === 0) {
      const row = { CDFILIAL: cdfilial };
      paramplaneja = await Http.post('/insertParamPlaneja', { requestType: 'Row', row });
      dados = paramplaneja.data.dataset.data;
    } else {
      dados = paramplaneja.data.data[0];
    }
    
    return dados;
  }
  
  public getParamPlanejaPadroes(cdfilial?: string) {
    const filter: IDictionary = [];
    const filial = (cdfilial === '' || cdfilial === undefined) ? [this.appController.cdCentral] : [cdfilial];
    this.onMountedFilterFind(filter, 'CDFILIAL', '=', filial);
    return Http.post('/find/paramPlanejaPadroes', { requestType: 'FilterData', filter }).then((response) => {
      const paramplaneja = response.data.data[0];
      return paramplaneja;
    });
  }
  
  public getCurrencySymbol(locale: string, currency: string) {
    return (0).toLocaleString(
      locale,
      {
        style: 'currency',
        currency,
        minimumFractionDigits: 0,
        maximumFractionDigits: 0,
      },
    ).replace(/\d/g, '').trim().concat(' ');
  }
  
  public getParamFilial(cdfilial: string) {
    const filter: IDictionary = [];
    this.onMountedFilterFind(
      filter,
      'CDFILIAL',
      '=',
      (cdfilial === '' || cdfilial === undefined) ? this.appController.cdCentral : cdfilial,
    );
    return Http.post('/find/paramfilial', { requestType: 'FilterData', filter }).then((response) => {
      const paramfilial = response.data.data[0];
      return paramfilial;
    });
  }
  
  // Exibe o alerta no topo da tela.
  public showAlert(name: string, error: any, color = 'green', time = 5000) {
    /*
    Foi removido a funcao Interceptor.addResponseFail de AppController.ts
    neste casom, tratamento de erros poderiam ter problemas, por isso foi
    realizado o tratamento abaixo, porém não houve situacao que fosse tratada
    pelo if, então este caso deve der acompanhado.
    */
    const text = error?.response?.data;
    let msg;
    
    if (text) {
      msg = I18n.translate(`pme::${text}`);
    } else {
      msg = I18n.translate(error);
    }
    
    const alertType = {
      name,
      color,
      text: msg,
      bottom: false,
      top: true,
      left: false,
      timeout: time,
      buttonsSlot: [
        {
          name: 'undo-button',
          component: 'ZdButton',
          icon: true,
          iconName: 'close',
          color: 'white',
          events: {
            click: () => { AlertService.hide(); },
          },
        },
      ],
    };
    AlertService.show(alertType);
  }

  public formatReportMask(mask: string) {
    return mask.replace(/[^(/\\.|\\)]/g, '@');
  }

  public getReportMaskNivel(mask: string, nivel: number): string {
    const arrayMask = mask.split('.');
    const newArrayMask = arrayMask.splice(0, nivel);
    const maskNivel = newArrayMask.join('.');
    return this.formatReportMask(maskNivel);
  }

  public timeIsValid(hour: string) {
    return /^([0-1]?[0-9]|2[0-4]):([0-5][0-9])(:[0-5][0-9])?$/.test(hour);
  }

  public formatHour(hour: string, twoPoints = false) {
    let formattedHour = null;
    if (hour) {
      if (!twoPoints) formattedHour = hour.replace(': ', '');
      else formattedHour = `${hour.substr(0, 2)}:${hour.substr(2, 2)}`;
    }
    return formattedHour;
  }
  
  public dateIsValid(dtIni: string, dtFin: string) {
    const format = 'DD/MM/YYYY';
    const dataIni = moment(dtIni, format, true);
    const dataFin = moment(dtFin, format, true);
    if (dataIni.isValid() && dataFin.isValid() && dataIni <= dataFin) return true;
    return false;
  }

  public prodIsValid(prodIni: string, prodFin: string) {
    const productMask: string = (Config.masks.productMask as PmeMask).pattern;
    const pinicial = Mask.getValueWithMask(prodIni, productMask);
    const pfinal = Mask.getValueWithMask(prodFin, productMask);
    
    const nvinicial = pinicial.length;
    const nvfinal = pfinal.length;
    let menornv;
    if (nvinicial > nvfinal) menornv = nvfinal;
    else menornv = nvinicial;

    let resultado = true;
    let i = 0;

    while (pinicial[i] === pfinal[i] && i < menornv) i += 1;

    if ((Object.prototype.hasOwnProperty.call(pinicial, i)
        && !Object.prototype.hasOwnProperty.call(pfinal, i))
        || (pinicial[i] > pfinal[i])
    ) {
      resultado = false;
    } else resultado = true;

    return resultado;
  }

  public calcTamacNivel(nivel: number, tipoMask = 'produto') {
    const tpMask: string = tipoMask === 'prato'
      ? (Config.masks.plateMask as PmeMask).pattern
      : (Config.masks.productMask as PmeMask).pattern;
    const mask = tpMask.split('.');
    let tamacnivel = 0;

    for (let i = 0; i < nivel; i += 1) {
      tamacnivel += mask[i].length;
    }
    return tamacnivel;
  }

  /**
   *  Funções do gridPeriodo.json
   */
  public setCdTipoCard(cdtipoCard: string) {
    this.cdtipoCard = cdtipoCard;
  }

  public setFilterModalPeriodo() {
    if (this.gridPeriodo !== undefined) {
      const conditions = [
        {
          name: 'P_CDFILCENTPME',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
        {
          name: 'P_CDTIPOCARD',
          operator: '=',
          value: this.cdtipoCard,
        },
      ];

      this.gridPeriodo.datasource.filter = conditions;
      this.gridPeriodo.datasource.get();
    }
  }
  
  public setCentral(central: string) {
    this.central = central;
  }
  
  public setFilterModalPeriodoImport() {
    if (this.gridPeriodoImport !== undefined) {
      const conditions = [
        {
          name: 'P_CDFILCENTPME',
          operator: '=',
          value: this.central,
        },
        {
          name: 'P_CDTIPOCARD',
          operator: '=',
          value: this.cdtipoCard,
        },
      ];

      this.gridPeriodoImport.datasource.filter = conditions;
      this.gridPeriodoImport.datasource.get();
    }
  }
  
  public setFilterModalPeriodoCentralModal(central: string, tipoCard: string) {
    if (this.gridPeriodo !== undefined || this.gridPeriodoImport !== undefined) {
      const conditions = [
        {
          name: 'P_CDFILCENTPME',
          operator: '=',
          value: central,
        },
        {
          name: 'P_CDTIPOCARD',
          operator: '=',
          value: tipoCard,
        },
      ];

      this.gridPeriodo.datasource.filter = conditions;
      this.gridPeriodo.datasource.get();
    }
  }

  public setGridInstance({ component }: IGridEvent) {
    this.gridPeriodo = component;
  }
  
  public setGridInstanceImport({ component }: IGridEvent) {
    this.gridPeriodoImport = component;
  }

  public setPeriodoParams({ row }: IGridEvent) {
    this.paramsCard = row;
    this.dtIniCard = row.DTINICARD;
    this.dtFinCard = row.DTFINCARD;
    this.dtIniPeriodo = row.DTINICARD;
    this.dtFinPeriodo = row.DTFINCARD;
    this.descCard = row.DSCARDPME;
    this.description = row.DSCARDPME;
    this.nrCardPme = row.NRCARDPME ? row.NRCARDPME : '';
    const modal = this.gridPeriodo.parent as Modal;
    modal.hide();
    // modal.destroy();
  }

  public clearPeriodoParams() {
    this.dtIniCard = '';
    this.dtFinCard = '';
    this.dtIniPeriodo = '';
    this.dtFinPeriodo = '';
    this.descCard = '';
    this.description = '';
    this.nrCardPme = '';
  }

  public getParamsCard() {
    return this.paramsCard;
  }
  
  public maskProd() {
    return (Config.masks.productMask as PmeMask).pattern;
  }
  
  public maskPlate() {
    return (Config.masks.plateMask as PmeMask).pattern;
  }
  
  public onMountedFilterFind(
    arr: IDictionary, nameColumn: string, operatorFind: string,
    valueColumn: string | number | null | undefined | IDictionary,
  ) {
    const object = {
      name: nameColumn,
      operator: operatorFind,
      value: valueColumn,
    };
    arr.push(object);
  }

  public setFilterProducts({ component }: IEventParam<Select>) {
    const nivelMax = (Config.masks.productMask as PmeMask).nivelMax;
    const conditions = [
      {
        name: 'MAXCDNVPRODUTO',
        operator: '=',
        value: nivelMax,
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public setDataText({ component }: IEventParam<Select>) {
    component.dataText = [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ];
  }

  public get setComponentPropsPrato() {
    return {
      component: 'ZdTextInput',
      mask: (Config.masks.plateMask as PmeMask).pattern,
    };
  }

  public get setComponentPropsProduto() {
    return {
      component: 'ZdTextInput',
      mask: (Config.masks.productMask as PmeMask).pattern,
    };
  }

  public pratoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDPRATO',
        mask: (Config.masks.plateMask as PmeMask).pattern,
      },
      'NMPRATO',
    ],
    formatterDataText: [
      {
        name: 'CDPRATO',
        mask: (Config.masks.plateMask as PmeMask).pattern,
      },
      'NMPRATO',
    ],
    dataValueOut: [
      {
        column: 'CDPRATO',
        columnOnGrid: 'CDPRATO',
      },
      {
        column: 'NMPRATO',
        columnOnGrid: 'NMPRATO',
      },
    ],
    dataValue: 'CDPRATO',
  };

  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
    ],
    dataValue: 'CDPRODUTO',
  };
  
  public getFormatLocaleWithWeek(): string {
    return (localStorage.getItem('LOGIN_LANGUAGE') !== 'en_us') ? 'DD/MM ddd' : 'MM/DD ddd';
  }
  
  public getFormatLocaleFullDataWithWeek(): string {
    return (localStorage.getItem('LOGIN_LANGUAGE') !== 'en_us') ? 'DD/MM/YYYY ddd' : 'MM/DD/YYYY ddd';
  }
  
  public getFormatLocale(): string {
    return (localStorage.getItem('LOGIN_LANGUAGE') !== 'en_us') ? 'DD/MM/YYYY' : 'MM/DD/YYYY';
  }
  
  get getDateFormat(): string {
    return (localStorage.getItem('LOGIN_LANGUAGE') !== 'en_us') ? 'DD/MM/YYYY' : 'MM/DD/YYYY';
  }

  public activeLoadingButton(button: Button) {
    const element = document.getElementsByName(button.name)[0] as HTMLElement;
    const progressEle = document.querySelectorAll('.loading-progress-line');
    progressEle.forEach((htmlElement: IDictionary) => {
      htmlElement.style.width = `${element.offsetWidth - 1}px`;
      htmlElement.style.left = `${element.offsetLeft}px`;
      // htmlElement.style.right = `${window.innerWidth - (element.offsetLeft + (element.offsetWidth * 2))}px`;
      htmlElement.style.top = `${element.offsetTop + (element.offsetHeight * 0.78)}px`;
      setTimeout(() => {
        htmlElement.style.visibility = 'visible';
      }, 50);
    });
    // const progressEle = document.getElementsByName(`${button.name}ProgressLine`)[0] as HTMLElement;
    button.disabled = true;
    button.cssClass = 'loadingButton';
  }

  public disableLoadingButton(button: Button) {
    const progressEle = document.querySelectorAll('.loading-progress-line');
    progressEle.forEach((htmlElement: IDictionary) => {
      htmlElement.style.visibility = 'hidden';
    });
    button.disabled = false;
    button.cssClass = '';
  }

  public centralIsValid() {
    const cdcentral = localStorage.getItem('CDFILIAL');
    let isValid = true;
    if (cdcentral === 'null' || !cdcentral) {
      isValid = false;
      DialogService.show(
        {
          name: 'paramCentral',
          type: 'warning',
          title: 'Atenção',
          text: 'A central de alimentação deve ser parametrizada.',
          buttons: [
            {
              name: 'dialogButton', label: 'OK', component: 'ZdButton', events: { click: () => DialogService.hide() },
            },
          ],
        },
      );
    }
    return isValid;
  }

  /**
   * Nome da barra de progresso deve seguir o formato : {nomeButton}Progress
   */
  public activeLoadingRoundButton(button: Button) {
    const progress = Metadata.getInstance<Progress>(`${button.name}Progress`);
    progress.cssClass = 'loading-progress-round';
    button.disabled = true;
  }

  public disableLoadingRoundButton(button: Button) {
    const progress = Metadata.getInstance<Progress>(`${button.name}Progress`);
    progress.cssClass = '';
    button.disabled = false;
  }

  public returnGrid(component: IDictionary) {
    let grid: IDictionary;
    if (component.type === 'button') grid = component.parent as IDictionary;
    else grid = component;
    return grid;
  }
  
  public validateNegative({ component }: IEventParam<TextInput>) {
    if (component.value < 0) component.value = '0';
  }
  
  public focusOnField(name: string) {
    const field = (document.getElementsByName(name)[0]) as HTMLElement;
    if (field) field.focus();
  }
  
  public formatValues(value: number, decimal: number): string {
    /* const numberMask = {
      decimalCharacter: ',',
      decimalCharacterAlternative: '.',
      digitGroupSeparator: '.',
      decimalPlaces: decimal,
    }; */
    const numberMask = (Config.masks.numberMask as PmeCurrencyMask);
    /* const separadores = localStorage.getItem('LOGIN_SEPARATOR') || '.,';
    const milharP = separadores.substring(0, 1);
    const decimalP = separadores.substring(2, 1); */
    
    if (decimal) numberMask.decimalPlaces = decimal;
    /*  numberMask.decimalCharacter = decimalP;
    numberMask.digitGroupSeparator = milharP; */
    
    const fn = FormatterParserProvider.getFormatter('ZdNumber');
    return fn(value, { mask: numberMask });
  }
  
  public removeDuplicateData(arr: IDictionary): IDictionary {
    return arr.filter((idx: number, i: number) => arr.indexOf(idx) === i);
  }
  
  public increamentZeroInHex(color: string) {
    let hexColor = color;
    for (let i = 0; i < (6 - color.length); i += 1) {
      hexColor += '0';
    }
    return hexColor;
  }

  public hexToRgb(hex: string) {
    const result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
    return result ? {
      r: parseInt(result[1], 16),
      g: parseInt(result[2], 16),
      b: parseInt(result[3], 16),
    } : null;
  }
  
  public hexToRgbInline(hex: string) {
    const result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
    const r = result ? parseInt(result[1], 16) : '';
    const g = result ? parseInt(result[2], 16) : '';
    const b = result ? parseInt(result[3], 16) : '';

    return `${r}.${g}.${b}`;
  }

  public RGBToTColor(r: any, g: any, b: any) {
    const TColor = r | (g << 8) | (b << 16); // eslint-disable-line
    return TColor;
  }
  
  public TColorToHex(TColor: any) {
    if (TColor) {
      const rgb = this.TColorToRGB(TColor);
  	  return this.rgbToHex(rgb); // eslint-disable-line
    } return '#FFFFFF';
  }

  public TColorToRGB(TColor: any) { // eslint-disable-line
    const r = TColor & 0xFF; // eslint-disable-line
    const g = (TColor >> 8) & 0xFF; // eslint-disable-line
    const b = (TColor >> 16) & 0xFF; // eslint-disable-line
    return [r, g, b]; // eslint-disable-line
  }

  public rgbToHex(rgb: IDictionary) {
    return '#' + ((1 << 24) + (rgb[0] << 16) + (rgb[1] << 8) + rgb[2]).toString(16).slice(1); // eslint-disable-line
  }
  
  // define a mascara de prato de acordo com o valor/padrao
  // a logica é percorrer o padrao até inserir todos os valores: podendo ser "." ou qualquer dígito "#"
  public getMaskPrato(value: string) {
    const maskPattern: string = (Config.masks.plateMask as PmeMask).pattern;
    const totalValue = value.length;
    let maskReturn = '';
    
    for (let i = 0, insertedNumbers = 0; insertedNumbers !== totalValue; i += 1) {
      if (maskPattern.charAt(i) === '.') {
        maskReturn = `${maskReturn}.`;
      } else {
        maskReturn = `${maskReturn}N`;
        insertedNumbers += 1;
      }
    }
    return maskReturn;
  }
  
  // define a mascara de produto de acordo com o valor/padrao
  // a logica é percorrer o padrao até inserir todos os valores: podendo ser "." ou qualquer dígito "#"
  public getMaskProduto(value: string) {
    const maskPattern: string = (Config.masks.productMask as PmeMask).pattern;
    const totalValue = value.length;
    let maskReturn = '';
    
    for (let i = 0, insertedNumbers = 0; insertedNumbers !== totalValue; i += 1) {
      if (maskPattern.charAt(i) === '.') {
        maskReturn = `${maskReturn}.`;
      } else {
        maskReturn = `${maskReturn}N`;
        insertedNumbers += 1;
      }
    }
    return maskReturn;
  }
  
  // mesma ideia que a funcao getMaskProduto, porem esta retorna o valor em si "1.01", e não de modo "N.NN"
  public getValueWithMaskProduto(value: string) {
    const maskPattern: string = (Config.masks.productMask as PmeMask).pattern;
    const totalValue = value.length;
    let maskReturn = '';
    
    for (let i = 0, insertedNumbers = 0; insertedNumbers !== totalValue; i += 1) {
      if (maskPattern.charAt(i) === '.') {
        maskReturn = `${maskReturn}.`;
      } else {
        maskReturn = `${maskReturn}${value.charAt(insertedNumbers)}`;
        insertedNumbers += 1;
      }
    }
    return maskReturn;
  }
  
  public getValueWithMaskPrato(value: string) {
    const maskPattern: string = (Config.masks.plateMask as PmeMask).pattern;
    const totalValue = value.length;
    let maskReturn = '';
    
    for (let i = 0, insertedNumbers = 0; insertedNumbers !== totalValue; i += 1) {
      if (maskPattern.charAt(i) === '.') {
        maskReturn = `${maskReturn}.`;
      } else {
        maskReturn = `${maskReturn}${value.charAt(insertedNumbers)}`;
        insertedNumbers += 1;
      }
    }
    return maskReturn;
  }

  public isSameOr(iniDate: IDictionary, endDate: IDictionary, type: 'before') {
    let result = false;
    if (type === 'before') result = iniDate.isBefore(endDate);
    else if (type === 'after') result = iniDate.isAfter(endDate);
    else result = iniDate.isSame(endDate);
    return result;
  }
  
  public setCentralDefault({ component }: IDictionary) {
    const central = this.appController.cdCentral;
    component.setValue(central);
  }
  
  public initalizeDtProcess() {
    this.getProcessDate('').then((processDate: string) => {
      this.dtProcessamento = processDate;
      localStorage.setItem('DTPROCESSAMENTO', processDate);
    });
  }
  
  public dateFormatWeekDay() {
    return `${this.getFormatLocale()} dddd`;
  }
  
  public scrollIntoElement(name: string) {
    const el = document.getElementsByClassName(name);
    if (el) el[0].scrollIntoView();
  }
  
  public setCurrentDate({ component }: IEventParam<ZdDate>) {
    const today = new Date().toLocaleDateString();
    const dtFormatVal = moment(today, 'DD/MM/YYYY').format(this.getFormatLocale());
    component.value = dtFormatVal;
  }
  
  public dragModal() {
    const modalElement: IDictionary = document.getElementsByClassName('draggableModal')[0];
    
    this.isDragging = !this.isDragging;
    
    modalElement.style.position = 'absolute';
    
    document.addEventListener('mousemove', (e: IDictionary) => {
      const event = window.event || e;
      
      if (this.isDragging) {
        // TODO: alterar valores para quando modal for fullscreen.
        const removePaddingLeft = 32 + 16;
        const removePaddingTop = 40;
        
        modalElement.style.left = `${event.clientX - modalElement.clientWidth + removePaddingLeft}px`;
        modalElement.style.top = `${event.clientY - removePaddingTop}px`;
      }
    });
  }
  
  public formatValuesToFixed(value: IDictionary, fixed: Number) {
    const separadores = localStorage.getItem('LOGIN_SEPARATOR') || '.,';
    // const milhar = separadores.substring(1);
    const decimal = separadores.substring(2, 1);
    
    if (fixed > 0 && decimal === ',') {
      return value.toFixed(fixed).replace('.', ',');
    }
    return value.toFixed(fixed);
  }
  
  public fillCode({ component }: IEventParam<TextInput>) {
    const size = component.validations.maxLength.limit;
    if (component.value < 0) component.value = '0';
    component.value = component.value.padStart(size, '0');
  }
  
  public returnColumn(component: GridEditable, nmColuna: string): Column | undefined {
    return component.columns.find((column: IDictionary) => column.name === nmColuna);
  }
  
  public validateVigenciaDates(result: IDictionary, ini: ZdDate, fim: ZdDate) {
    let valid = true;
    result.forEach((vigencia: IDictionary) => {
      const dtvigencia = vigencia.DTVIGENCIA ? moment(vigencia.DTVIGENCIA, 'DD/MM/YYYY') : vigencia.DTVIGENCIA;
      const dtvigenciafim = vigencia.DTVIGENCIAFIM ? moment(vigencia.DTVIGENCIAFIM, 'DD/MM/YYYY') : vigencia.DTVIGENCIAFIM;
      const iniValue = ini.value ? moment(ini.value, 'DD/MM/YYYY') : ini.value;
      const fimValue = fim.value ? moment(fim.value, 'DD/MM/YYYY') : fim.value;
      if ((dtvigencia <= iniValue && iniValue <= dtvigenciafim) || (dtvigencia <= fimValue && fimValue <= dtvigenciafim)) {
        valid = false;
      }
      if (dtvigencia.diff(iniValue, 'days') === 0) valid = false;
    });
    return valid;
  }
  
  public isMerenda() {
    /* 4875 produto Alimentacao Escolar Next
    6935 produto Alimentacao Escolar Angular */
    return localStorage.getItem('PRODUCT_ID') === '4875' || localStorage.getItem('PRODUCT_ID') === '6935';
  }
  
  public getCurrencySymbolLog() {
    return (Config.masks.currencyMask as PmeCurrencyMask).currencySymbol;
  }
  
  public roundFloat(value: number, decimals: number) {
    // if (Number.isNaN(value)) return value;
    // if (!decimals) decimals = 0;
    return Math.round(value * (10 ** decimals)) / (10 ** decimals);
  }
  
  public merge(arr1: any, arr2: any, predicate = (key1: any, key2: any) => key1 === key2) {
    const merged = [...arr1];
    arr2.forEach((arr2El: any) => (merged.some((mergedEl) => predicate(arr2El, mergedEl)) ? null : merged.push(arr2El)));
    return merged;
  }
  
  public checkOpenModalOverlap() {
    const modals = ModalService.modals;
    const visibleModals = modals.filter((el: IDictionary) => el.isVisible === true);
    return visibleModals.length === 0;
  }
  
  public doubleClick(dblclickFn: Function, event: Event) {
    if (this.lastCellClicked && this.lastCellClicked !== event.currentTarget) {
      this.clicks = 0;
    }
  
    this.clicks += 1;
    this.lastCellClicked = event.currentTarget;
  
    if (this.clicks === 1) {
      setTimeout(() => {
        if (this.clicks === 2) {
          dblclickFn();
        }
        this.clicks = 0;
      }, 300); // Tempo em milissegundos para esperar pelo segundo clique
    }
  }
  
  // função para checar nos grid editables que tem a propriedade doubleClickEdit ativada, se o click não foi duplo
  public notDoubleClick(dblclickFn: Function, event: Event) {
    if (this.lastCellClicked && this.lastCellClicked !== event.currentTarget) {
      this.clicks = 0;
    }
  
    this.clicks += 1;
    this.lastCellClicked = event.currentTarget;
  
    if (this.clicks === 1) {
      setTimeout(() => {
        if (this.clicks === 1) {
          dblclickFn();
        }
        this.clicks = 0;
      }, 300); // Tempo em milissegundos para esperar pelo segundo clique
    }
  }
  
  /* eslint-disable */
  public caracteresSemAcento = {
    À: 'A', Á: 'A', Â: 'A', Ã: 'A', Ä: 'A', Å: 'A', Æ: 'A', à: 'a', á: 'a', â: 'a', ã: 'a', ä: 'a', å: 'a', æ: 'a',
    È: 'E', É: 'E', Ê: 'E', Ë: 'E', è: 'e', é: 'e', ê: 'e', ë: 'e',
    Ì: 'I', Í: 'I', Î: 'I', Ï: 'I', ì: 'i', í: 'i', ï: 'i', î: 'i',
    Ò: 'O', Ó: 'O', Ô: 'O', Õ: 'O', Ö: 'O', Ø: 'O', ð: 'o', ò: 'o', ó: 'o', ô: 'o', õ: 'o', ö: 'o', ø: 'o',
    Ù: 'U', Ú: 'U', Û: 'U', Ü: 'U', ù: 'u', ú: 'u', û: 'u', ü: 'u',
    Ñ: 'N', ñ: 'n', ç: 'c', Ç: 'Ç',
  };
  /* eslint-enable */
  
  public strtr(inputStr: string, valuesParam?: IDictionary) {
    let str = inputStr;
    let values;
    if (!valuesParam) values = this.caracteresSemAcento;
    else values = valuesParam;
    Object.entries(values).forEach((el: any) => {
      str = str.replace(el[0], el[1]);
    });
    return str;
  }
  
  public hideLoading() {
    LoadingService.hide();
  }
  
  public addFilter(datasource: IDatasource, column: string, operator: string, value: string) {
    const newFilter = [{
      name: column,
      operator,
      value,
    }];
    return Array.isArray(datasource?.filter)
      ? this.merge(newFilter, datasource?.filter, (a: any, b: any) => a.name === b.name)
      : this.merge(newFilter, [datasource?.filter], (a: any, b: any) => a.name === b.name);
  }
  
  // private merge(a: any, b: any, predicate = (a, b) => a === b) {
  //   const c = [...a]; // copy to avoid side effects
  //   // add all items from B to copy C if they're not already present
  //   b.forEach((bItem) => (c.some((cItem) => predicate(bItem, cItem)) ? null : c.push(bItem)));
  //   return c;
  // }
}
